/*
 * ***** BEGIN LICENSE BLOCK *****
 * Copyright (C) 2011-2016 ZeXtras
 *
 * The contents of this file are subject to the ZeXtras EULA;
 * you may not use this file except in compliance with the EULA.
 * You may obtain a copy of the EULA at
 * http://www.zextras.com/zextras-eula.html
 * ***** END LICENSE BLOCK *****
 */

function ZxLicense(licenseCallback) {
  this._infos = {};
  this._delegatedAdminChecked = false;
  this._callback = licenseCallback;
  this._loadAction = new AjxTimedAction(this, this._loadLicense, {});
  this._errorNotified = false;
  this._loadLicense();
}

ZxLicense.WHITE_LIST_LICENSING = [
  'ZxCore',
  'ZxChat',
  'ZxNotifications'
];
ZxLicense.WHITE_LIST_ENABLED = [
  'ZxCore',
  'ZxMig',
  'ZxNotifications'
];

ZxLicense._DEFAULT_DATA = {
  accountCount: 0,
  company: '',
  customer: '',
  dateEnd: 0,
  dateStart: 0,
  maintenanceEnd: 0,
  expired: false,
  notYetValid: false,
  ispLegacy: false,
  licensedUsers: 0,
  modules: {
    ZxAdmin: {
      enabled: false,
      licensed: false,
      quantity: 0
    },
    ZxBackup: {
      enabled: false,
      licensed: false,
      quantity: 0
    },
    ZxChat: {
      enabled: false,
      licensed: false,
      quantity: 0
    },
    ZxMobile: {
      enabled: false,
      licensed: false,
      quantity: 0
    },
    ZxPowerstore: {
      enabled: false,
      licensed: false,
      quantity: 0
    }
  },
  order_id: '',
  serverID: '00000000-0000-0000-0000-000000000000',
  reseller: false,
  resourceCount: 0,
  type: ''
};

/**
 * Link to the store of ZeXtras
 * @type {string}
 */
ZxLicense.zextrasStore = ZxLinks.Store;
ZxLicense.support = ZxLinks.Support;

ZxLicense.LICENSE_NONE = 'None';
ZxLicense.LICENSE_PERPETUAL = 'Perpetual';
ZxLicense.LICENSE_PERPETUAL_V2 = 'Perpetual v2';
ZxLicense.LICENSE_PURCHASED = 'Purchased';
ZxLicense.LICENSE_TRIAL = 'Trial';
ZxLicense.LICENSE_ISP = 'ISP';
ZxLicense.LICENSE_NULL = '';
ZxLicense.LICENSE_MIG = 'ZxMig';

ZxLicense._KEY_INTO_GRACE_LIMIT = 'isWithinGraceInterval';

/**
 * Reload time for License informations
 * @type {number}
 */
ZxLicense.LOAD_TIME = 10 * 60 * 1000;

/**
 * Get if the license is under quota.
 * @param {string} server
 * @return {boolean}
 */
ZxLicense.prototype.isUserExceeded = function(server) {
  if (!this.hasServerLicenseInfo(server)) {
    return false;
  }
  if (this.getServerLicenseInfo(server).licensedUsers === 'unlimited') {
    return false;
  }

  if (this.getServerLicenseInfo(server).licensedUsers === 'none') {
    return false;
  }
  return this.getServerLicenseInfo(server).accountCount > this.getServerLicenseInfo(server).licensedUsers;
};

/**
 * Geti if the license type is 'purchased'
 * @param {string} server
 * @return {boolean}
 */
ZxLicense.prototype.isPurchased = function(server) {
  if (!this.hasServerLicenseInfo(server)) {
    return false;
  }
  return this.getServerLicenseInfo(server).type === ZxLicense.LICENSE_PURCHASED;
};

/**
 * Geti if the license type is 'perpetual'
 * @param {string} server
 * @return {boolean}
 */
ZxLicense.prototype.isPerpetual = function(server) {
  if (!this.hasServerLicenseInfo(server)) {
    return false;
  }
  return this.getServerLicenseInfo(server).type === ZxLicense.LICENSE_PERPETUAL;
};

/**
 * Geti if the license type is 'perpetual v2'
 * @param {string} server
 * @return {boolean}
 */
ZxLicense.prototype.isPerpetualV2 = function(server) {
  if (!this.hasServerLicenseInfo(server)) {
    return false;
  }
  return this.getServerLicenseInfo(server).type === ZxLicense.LICENSE_PERPETUAL_V2;
};

/**
 * return true if the license type is 'none'
 * @param {string} server
 * @return {boolean}
 */
ZxLicense.prototype.isEmptyLicense = function(server) {
  if (!this.hasServerLicenseInfo(server)) {
    return false;
  }
  return this.getServerLicenseInfo(server).type === ZxLicense.LICENSE_NONE;
};

/**
 * Get if the license is about to expire.
 * @param {string} server
 * @return {boolean}
 */
ZxLicense.prototype.isAboutToExpire = function(server) {
  if (this.isPerpetual(server)) {
    return false;
  }

  var now = new Date().getTime(),
    end = this.getDateEnd(server).getTime();

  return (end - now <= ZxUtils.DaysToMillis(7));
};

/**
 * Get if the license is expired.
 * @param {string} server
 * @return {boolean}
 */
ZxLicense.prototype.isExpired = function(server) {
  if (!this.hasServerLicenseInfo(server)) {
    return false;
  }
  return this.getServerLicenseInfo(server).expired;
};

/**
 * Get if the license is ISP Version 2.
 * @param {string} server
 * @return {boolean}
 */
ZxLicense.prototype.isISPV2 = function(server) {
  if (!this.hasServerLicenseInfo(server)) {
    return false;
  }
  return this.getServerLicenseInfo(server).ispLegacy;
};


/**
 * Get if the license is not yet valid.
 * @param {string} server
 * @return {boolean}
 */
ZxLicense.prototype.isNotYetValid = function(server) {
  if (!this.hasServerLicenseInfo(server)) {
    return false;
  }
  return this.getServerLicenseInfo(server).notYetValid;
};

/**
 * Get if the license is from a reseller.
 * @param {string} server
 * @return {boolean}
 */
ZxLicense.prototype.isReseller = function(server) {
  if (!this.hasServerLicenseInfo(server)) {
    return false;
  }
  return this.getServerLicenseInfo(server).reseller === true &&
    this.getServerLicenseInfo(server).type !== ZxLicense.LICENSE_TRIAL;
};

/**
 * Get if the license is expired.
 * If server name is not provided, will return true if at least one is trial (should not happen).
 * @param {string=} server
 * @return {boolean}
 */
ZxLicense.prototype.isTrial = function(server) {
  if (server) {
    if (!this.hasServerLicenseInfo(server)) {
      return false;
    }
    return this.getServerLicenseInfo(server).type === ZxLicense.LICENSE_TRIAL;
  } else {
    var srvId, oneTrial = false;
    for (srvId in this._infos) {
      if (!this._infos.hasOwnProperty(srvId)) { continue; }
      oneTrial = oneTrial || this._infos[srvId].type === ZxLicense.LICENSE_TRIAL;
    }
    return oneTrial;
  }
};

/**
 * Get if the license is paid for at least one module.
 * @param {string=} server
 * @return {boolean}
 */
ZxLicense.prototype.canRemoveChatBrand = function(server) {
  if (server) {
    if (!this.hasServerLicenseInfo(server)) {
      return false;
    }
    return this.getServerLicenseInfo(server).canRemoveChatBrand;
  } else {
    var srvId, oneTrial = false;
    for (srvId in this._infos) {
      if (!this._infos.hasOwnProperty(srvId)) { continue; }
      oneTrial = oneTrial || this._infos[srvId].canRemoveChatBrand;
    }
    return oneTrial;
  }
};

/**
 * Get if the license is ISP.
 * @param {string} server
 * @return {boolean}
 */
ZxLicense.prototype.isISP = function(server) {
  if (!this.hasServerLicenseInfo(server)) {
    return false;
  }
  return this.getServerLicenseInfo(server).type === ZxLicense.LICENSE_ISP;
};

/**
 * Get the last online validation of the ISP License.
 * @param {string} server
 * @return {Date}
 */
ZxLicense.prototype.getLastValidation = function(server) {
  if (!this.isISP(server)) { return new Date(0); }
  return new Date(this.getServerLicenseInfo(server).lastLicenseValidation);
};

/**
 * Get how many days the ISP License will be active.
 * @param {string} server
 * @return {number}
 */
ZxLicense.prototype.getRenewTimeLeft = function(server) {
  if (!this.isISP(server)) { return 0; }
  return this.getServerLicenseInfo(server).renewTimeLeft;
};

/**
 * Get the how many days the ISP License.will be active.
 * @param {string} server
 * @return {boolean}
 */
ZxLicense.prototype.hideRenewTimeLeft = function(server) {
  if (!this.isISP(server)) { return true; }
  return this.getServerLicenseInfo(server).withinSilentWarningInterval;
};

/**
 * Get if the license is ISP.
 * @param {string} server
 * @return {string}
 */
ZxLicense.prototype.getServerID = function(server) {
  return this.getServerLicenseInfo(server).serverID;
};

/**
 * Get the license type.
 * @param {string} server
 * @return {string}
 */
ZxLicense.prototype.getType = function(server) {
  if (!this.hasServerLicenseInfo(server)) {
    return ZxLicense.LICENSE_NULL;
  }
  return this.getServerLicenseInfo(server).type;
};

/**
 * Get the licensed modules.
 * @param {string} server
 * @return {{}}
 */
ZxLicense.prototype.getModules = function(server) {
  return this.getServerLicenseInfo(server).modules;
};

/**
 * Get the licensed company.
 * @param {string} server
 * @return {string}
 */
ZxLicense.prototype.getCompany = function(server) {
  return this.getServerLicenseInfo(server).company;
};

/**
 * Get the licensed customer.
 * @param {string} server
 * @return {string}
 */
ZxLicense.prototype.getCustomer = function(server) {
  return this.getServerLicenseInfo(server).customer;
};

/**
 * Get the licensed reseller.
 * @param {string} server
 * @return {boolean}
 */
ZxLicense.prototype.getReseller = function(server) {
  return this.getServerLicenseInfo(server).reseller;
};

/**
 * Get the Start date of the license.
 * @param {string} server
 * @return {Date}
 */
ZxLicense.prototype.getDateStart = function(server) {
  return new Date(this.getServerLicenseInfo(server).dateStart);
};

/**
 * Get the end date of the license.
 * @param {string} server
 * @return {Date}
 */
ZxLicense.prototype.getDateEnd = function(server) {
  return new Date(this.getServerLicenseInfo(server).dateEnd);
};

/**
 * Get the end date of the maintenance.
 * @param {string} server
 * @return {Date}
 */
ZxLicense.prototype.getDateEndMaintenance = function(server) {
  return new Date(this.getServerLicenseInfo(server).maintenanceEnd);
};

/**
 * Get the order ID of the license.
 * @param {string} server
 * @return {string}
 */
ZxLicense.prototype.getOrderID = function(server) {
  return this.getServerLicenseInfo(server).order_id;
};

/**
 * Get the number of the licensed users.
 * @param {string} server
 * @return {number}
 */
ZxLicense.prototype.getLicensedUsers = function(server) {
  return this.getServerLicenseInfo(server).licensedUsers || 0;
};

/**
 * Get the count of the accounts.
 * @param {string} server
 * @return {number}
 */
ZxLicense.prototype.getCountAccount = function(server) {
  return this.getServerLicenseInfo(server).accountCount || 0;
};

/**
 * Get the count of the resources account.
 * @param {string} server
 * @return {number}
 */
ZxLicense.prototype.getCountResource = function(server) {
  return this.getServerLicenseInfo(server).resourceCount || 0;
};

/**
 * Get the total count.
 * @param {string} server
 * @return {number}
 */
ZxLicense.prototype.getCountTotal = function(server) {
  return this.getCountResource(server) + this.getCountAccount(server);
};

/**
 * Get the licensed Zimbra version.
 * @param {string} server
 * @return {string|undefined}
 */
ZxLicense.prototype.getLicensedZimbraVersion = function(server) {
  var info = this.getServerLicenseInfo(server);
  if (info.zimbraVersion) {
    return [
      info.zimbraVersion.major,
      info.zimbraVersion.minor,
      info.zimbraVersion.micro
    ].join('.');
  }
};

/**
 * Load the license informations from the server.
 * The loading is asynchronous.
 */
ZxLicense.prototype._loadLicense = function() {
  if (this._scheduledActionID) {
    AjxTimedAction.cancelAction(this._scheduledActionID);
    delete this._scheduledActionID;
  }

  if (!ZeXtrasItem.isUserAdmin() && !this._delegatedAdminChecked) {
    this._delegatedAdminChecked = true;
    var resp,
      tmpResp = ZxAdminRequest.sendSyncRequest(
        ZxAdminRequest.TARGET_LOCAL,
        'ZxCore',
        'getLicenseInfo'
      );

    resp = tmpResp[ZxAdminRequest.TARGET_LOCAL];
    if (resp.ok && resp.response) {
      var licensed = resp.response.modules['ZxAdmin'].licensed;

      if (!licensed) {
        var err = new Error(zxMsg('MSG_LIC_Unlicensed_delegated'));
        err.code = 'LIC_1';
        throw err;
      }
    }
  }

  ZxAdminRequest.sendRequest(
    ZxAdminRequest.TARGET_ALL,
    'ZxCore',
    'getLicenseInfo'
  ).then(
    AjxCallback.simpleClosure(
      function(resp) {
        ZxUtils.log(resp, 'License Informations');
        this._scheduledActionID = AjxTimedAction.scheduleAction(this._loadAction, ZxLicense.LOAD_TIME);
        var oldInfo = ZxUtils.cloneObj(this._infos),
          server,
          infoDiff;
        this._infos = {};
        for (server in resp) {
          if (!resp.hasOwnProperty(server)) { continue; }
          if (!resp[server].ok) {
            this._infos[server] = ZxLicense._DEFAULT_DATA;
            continue;
          }
          this._infos[server] = resp[server].response;
        }
        infoDiff = ZxUtils.objDiff(this._infos, oldInfo);
        if (!ZxUtils.isObjectEmpty(infoDiff)) {
          this._notifyChanges();
        }
      },
      this
    ),
    AjxCallback.simpleClosure(
      this._catchLoadErrors,
      this
    )
  );
};

/**
 * Notify new license informations are available.
 */
ZxLicense.prototype._notifyChanges = function() {
  this._callback.run(this);
};

ZxLicense.prototype.getSalesLink = function(server) {
  return ZxLicense.support;
};

ZxLicense.prototype.getNewLicenseLink = function(server) {
  return ZxLicense.zextrasStore;
};

ZxLicense.prototype.getUpgradeLink = function(server) {
  return ZxLicense.zextrasStore;
};

/**
 * Get the warning text to display into the core sections head.
 * @param {string} server
 * @param {string=} moduleId
 * @param {string=} moduleName
 * @return {string}
 */
ZxLicense.prototype.getWarningText = function(server, moduleId, moduleName) {
  if (ZxServer.getAll().getArray().length < 1) {
    return '';
  }

  var text = '',
    new_link = this.getNewLicenseLink(server),
    upgrade_link = this.getUpgradeLink(server),
    sales_link = this.getSalesLink(server);

  if (this.isNotYetValid(server)) {
    return zxMsg('MSG_COR_license_not_yet_valid');
  }

  if (this.isExpired(server)) {
    if (this.isReseller(server)) {
      text = zxMsg('MSG_COR_license_expired_contact_reseller');
    } else {
      if (this.isTrial(server)) {
        text = zxMsg(
          'MSG_COR_license_expired_trial',
          ['<strong><a href ="' + new_link + '" target="_blank">', '</a></strong>',
            '<strong><a href="' + sales_link + '" target="_blank">']
        );
      } else {
        text = zxMsg(
          'MSG_COR_license_expired',
          ['<strong><a href ="' + new_link + '" target="_blank">', '</a></strong>']
        );
      }
    }
    return text;
  }

  if (this.isUserExceeded(server)) {
    if (this.isReseller(server)) {
      text = zxMsg('MSG_COR_license_quota_exceed_contact_reseller');
    } else {
      if (this.isTrial(server)) {
        text = zxMsg(
          'MSG_COR_license_trial_exceed_contact_sales',
          ['<strong><a href ="' + sales_link + '" target="_blank">', '</a></strong>']
        );
      } else {
        text = zxMsg(
          'MSG_COR_license_trial_exceed_upgrade',
          ['<strong><a href ="' + upgrade_link + '" target="_blank">', '</a></strong>']
        );
      }
    }
    return text;
  }

  if ( (moduleId == null || this.isModuleLicensed(moduleId, server))
       && this.isAboutToExpire(server) && !this.isEmptyLicense(server) && !this.isPerpetualV2(server)) {
    if (this.isReseller(server)) {
      text = zxMsg('MSG_COR_license_about_expire_contact_reseller');
    } else {
      if (this.isTrial(server)) {
        text = zxMsg(
          'MSG_COR_license_about_expire_upgrade',
          ['<strong><a href ="' + upgrade_link + '" target="_blank">', '</a></strong>']
        );
      } else {
        text = zxMsg(
          'MSG_COR_license_about_expire_upgrade_info',
          ['<strong><a href ="' + upgrade_link + '" target="_blank">', '</a></strong>']
        );
      }
    }
    return text;
  }

  if (this.isTrial(server)) {
    text = zxMsg(
      'MSG_COR_license_trial_message',
      ['<strong><a href="' + ZxLinks.COR_License_Management_info + '" target="_blank">', '</a></strong>']
    );
    return text;
  }

  if (moduleId && !this.isModuleLicensed(moduleId, server)) {
    if (this.isReseller(server)) {
      text = zxMsg('MSG_COR_Module_not_licensed_contact_reseller', [moduleName]);
    } else {
      text = zxMsg(
        'MSG_COR_Module_not_licensed_upgrade',
        [moduleName, '<strong><a href ="' + upgrade_link + '" target="_blank">', '</a></strong>']
      );
    }
    return text;
  }

  return text;
};

/**
 * Return the server license information.
 * @param {string} server
 * @return {*}
 */
ZxLicense.prototype.getServerLicenseInfo = function(server) {
  if (this.hasServerLicenseInfo(server)) {
    return this._infos[server];
  } else {
    return ZxLicense._DEFAULT_DATA
  }
};

/**
 * Return if the license information are laded for a specific server.
 * @param {server} server
 * @return {boolean}
 */
ZxLicense.prototype.hasServerLicenseInfo = function(server) {
  return this._infos.hasOwnProperty(server);
};

/**
 * Invalidate the current license information, forcing to reload the information from the server.
 */
ZxLicense.prototype.invalidate = function() {
  this._loadLicense();
};

/**
 * Get if the license for the specified server is into the grace limit.
 * The information is part of the license and are calculated on server side.
 * (Usually 5% up to 20 account)
 * @param {string} server
 * @return {boolean}
 */
ZxLicense.prototype.isWithinGraceInterval = function(server) {
  var licensed = false,
    i;
  if (server === ZxAdminRequest.TARGET_LOCAL ||
      server === ZxAdminRequest.TARGET_ALL) {
    for (i in this._infos) {
      if (!this._infos.hasOwnProperty(i)) { continue; }
      licensed = licensed || this._infos[i][ZxLicense._KEY_INTO_GRACE_LIMIT];
    }
  } else {
    licensed = licensed || this._infos[server][ZxLicense._KEY_INTO_GRACE_LIMIT];
  }
  return licensed;
};

/**
 * Return if the module is licensed for the specified server.
 * @param {string} module
 * @param {string=} server
 * @return {boolean}
 */
ZxLicense.isModuleLicensed = function(module, server) {
  if (ZxUtils.indexOf(ZxLicense.WHITE_LIST_LICENSING, module) > -1) { return true; }
  if (typeof ZeXtras === 'undefined') { return false; }
  return ZeXtras.getLicense().isModuleLicensed(module, server || ZxAdminRequest.TARGET_LOCAL);
};

/**
 * Return if the module id licensed for the specified server.
 * @param {string} module
 * @param {string=} server - Server to check, if not specified, ZxAdminRequest.TARGET_LOCAL will be used.
 * @return {boolean}
 */
ZxLicense.prototype.isModuleLicensed = function(module, server) {
  if (ZxUtils.indexOf(ZxLicense.WHITE_LIST_LICENSING, module) > -1) { return true; }

  server = server || ZxAdminRequest.TARGET_LOCAL;

  var modInfo,
    serverName;

  if (server === ZxAdminRequest.TARGET_LOCAL ||
      server === ZxAdminRequest.TARGET_ALL) {

    for (serverName in this._infos) {
      if (!this._infos.hasOwnProperty(serverName)) { continue; }

      if (module === 'ZxMig' && this.getType(serverName) === ZxLicense.LICENSE_MIG) {
        return true;
      }

      if (!this.getModules(serverName).hasOwnProperty(module)) { continue; }

      if (this.getModules(serverName)[module].licensed === true) {
        return true;
      }
    }

  } else {

    if (module === 'ZxMig' && this.getType(server) === ZxLicense.LICENSE_MIG) {
      return true;
    }

    if (this.getModules(server) && this.getModules(server).hasOwnProperty(module)) {
      modInfo = this.getModules(server)[module];
      if (modInfo.hasOwnProperty('licensed')) {
        return modInfo.licensed;
      }
    }

  }
  return false;
};

/**
 * Return if the module is licensed for the specified server.
 * @param {string} module
 * @param {string=} server
 * @return {boolean}
 */
ZxLicense.isModuleEnabled = function(module, server) {
  if (ZxUtils.indexOf(ZxLicense.WHITE_LIST_ENABLED, module) > -1) { return true; }
  return ZeXtras.getLicense().isModuleEnabled(module, server || ZxAdminRequest.TARGET_LOCAL);
};

/**
 * Return if the module is enabled on the specified server.
 * @param {string} module
 * @param {string=} server - Server to check, if not specified, ZxAdminRequest.TARGET_LOCAL will be used.
 * @return {boolean}
 */
ZxLicense.prototype.isModuleEnabled = function(module, server) {
  if (ZxUtils.indexOf(ZxLicense.WHITE_LIST_ENABLED, module) > -1) { return true; }

  server = server || ZxAdminRequest.TARGET_LOCAL;

  var modInfo,
    serverName;

  if (server === ZxAdminRequest.TARGET_LOCAL ||
      server === ZxAdminRequest.TARGET_ALL) {

    for (serverName in this._infos) {
      if (!this._infos.hasOwnProperty(serverName)) { continue; }

      if (module === 'ZxMig' && this.getType(serverName) === ZxLicense.LICENSE_MIG) {
        return true;
      }

      if (!this.getModules(serverName).hasOwnProperty(module)) { continue; }

      if (this.getModules(serverName)[module].enabled === true) {
        return true;
      }
    }

  } else {

    if (module === 'ZxMig' && this.getType(server) === ZxLicense.LICENSE_MIG) {
      return true;
    }
    if (module === 'ZxCore' && this.getType(server) !== ZxLicense.LICENSE_MIG) {
      return true;
    }

    if (this.getModules(server) && this.getModules(server).hasOwnProperty(module)) {
      modInfo = this.getModules(server)[module];
      if (modInfo.hasOwnProperty('enabled')) {
        return modInfo.enabled;
      }
    }

  }
  return false;
};

ZxLicense.prototype._catchLoadErrors = function(err) {
  if (!(typeof err.getCode === 'function' &&
    err.getCode() === ZxErrorCode.ZM_CSFE_EXCEPTION &&
    typeof err.getDetail === 'function' &&
    err.getDetail(ZxSOAPError.KEY_CODE) &&
    err.getDetail(ZxSOAPError.KEY_CODE) === ZmCsfeException.EMPTY_RESPONSE) &&
    !this._errorNotified) {
    this._errorNotified = true;
    ZxUtils.catchError(err);
  }
  this._scheduledActionID = AjxTimedAction.scheduleAction(this._loadAction, ZxLicense.LOAD_TIME);
  ZxUtils.log(err, 'Error during loading license informations');
  return true;
};

/**
 * Return the message shown when new license is uploaded.
 * @param {[]} propertyKeys
 * @param {{}} templateMap
 * @return {string}
 */
ZxLicense.buildUploadWarningMessage = function(propertyKeys, templateMap) {
  var warningString = ['<ul>'], numbershown = false, i;

  for (i in propertyKeys) {
    if (!propertyKeys.hasOwnProperty(i)) {
      continue;
    }

    switch (propertyKeys[i]) {
    case 'maintenance_period_expired':
      warningString.push('<li>' + zxMsg('MSG_LIC_Maintenance_period_expired') + '</li>');
      break;

    case 'failed_blacklisted':
      warningString.push('<li>' + zxMsg('MSG_LIC_License_voided') + '</li>');
      break;

    case 'upload_failed_expired':
      warningString.push('<li>' + zxMsg('MSG_LIC_Try_upload_expired_license') + '</li>');
      break;

    case 'failed_renew':
      warningString.push('<li>' + zxMsg('MSG_LIC_Failed_ISP_audit', [templateMap.serverName]) + '</li>');
      break;

    case 'warning_users':
    case 'failed_users':
      if (numbershown) { break; }
      numbershown = true;
      warningString.push('<li>' + zxMsg('MSG_LIC_Failed_too_many_users',
        [templateMap.globalAccountNumber, templateMap.licensedUsers]) + '</li>');
      break;

    case 'warning_expire':
      warningString.push('<li>' + zxMsg('MSG_LIC_License_about_to_expire', [templateMap.daysToExpiration]) + '</li>');
      break;

    case 'warning_modules_not_licensed_anymore':
      warningString.push('<li>' + zxMsg('MSG_LIC_Modules_will_be_disabled', [templateMap.missingModules]) + '</li>');
      break;

    case 'warning_older_license':
      warningString.push('<li>' + zxMsg('MSG_LIC_Try_upload_older_license') + '</li>');
      break;

    case 'warning_renew':
      warningString.push('<li>' + zxMsg('MSG_LIC_License_will_be_renewed_in_n_days') + '</li>');
      break;

    case 'warning_renew_contract':
      warningString.push('<li>' + zxMsg('MSG_LIC_Failed_ISP_audit', [templateMap.serverName]) + '</li>');
      break;

    case 'warning_replace_perpetual':
      warningString.push('<li>' + zxMsg('MSG_LIC_Try_replace_perpetual_license') + '</li>');
      break;

    default:
      continue;
    }
  }
  warningString.push('</ul>');

  return warningString.join('');
};
