/*
 * ***** BEGIN LICENSE BLOCK *****
 * Copyright (C) 2011-2016 ZeXtras
 *
 * The contents of this file are subject to the ZeXtras EULA;
 * you may not use this file except in compliance with the EULA.
 * You may obtain a copy of the EULA at
 * http://www.zextras.com/zextras-eula.html
 * ***** END LICENSE BLOCK *****
 */

/**
 * Container for the operations
 * @param {string} server
 * @param {string} module
 * @param {string} opName
 * @param {{}} opData
 * @constructor
 */
function ZxOperation(opData) {
  this.opData = opData;
  this.server = opData[ZxOperation.KEY_HOST];
  this.opName = opData[ZxOperation.KEY_NAME];
  this.module = opData[ZxOperation.KEY_MODULE];
  this._taskItem = null;

  this.uuid = this._uuid = opData[ZxOperation.KEY_ID];

  if (typeof ZaTask !== 'undefined') {
    // viewForPopup, cacheName, title, data, position, type
    // displayName, finishCallback, cacheDialog, dialogType, editData
    var icon = this.module,
      dialog;

    if (ZeXtras.isZxMig(this.server)) {
      icon = 'ZxMig';
    }

    if (ZxOperation.getTaskItemTypeCode(opData.state) === 1) {
      dialog = ZxOperationDialog;
    } else if (ZxOperation.getTaskItemTypeCode(opData.state) === 2) {
      dialog = new ZxOperationDialog(ZaApp.getInstance().getAppCtxt().getShell(), this);
      dialog.setObject(this);
    }

    this._taskItem = new ZaTaskItem(
      dialog,
      this.opData.id,
      this.opData.name,
      this,
      new DwtRectangle(),
      ZxOperation.getTaskItemTypeCode(opData.state),    // 1 = working, 2 = planned
      ZxUtils.getTitleWithIcon(icon, this.opData.descName)
    );
  }
}

ZxOperation.KEY_MODULE = 'module';
ZxOperation.KEY_NAME = 'name';
ZxOperation.KEY_DESC_NAME = 'descName';
ZxOperation.KEY_ID = 'id';
ZxOperation.KEY_STATE = 'state';
ZxOperation.KEY_HOST = 'host';
ZxOperation.KEY_START_TIME = 'startTime';
ZxOperation.KEY_QUEUED_TIME = 'queuedTime';

/**
 * Function used to order the operation's array.
 * @param {ZxOperation} a
 * @param {ZxOperation} b
 * @return {number}
 */
ZxOperation.ORDER_BY_DATE = function(a, b) {
  if (a.opData[ZxOperation.KEY_START_TIME] < b.opData[ZxOperation.KEY_START_TIME]) {
    return -1;
  } else if (a.opData[ZxOperation.KEY_START_TIME] > b.opData[ZxOperation.KEY_START_TIME]) {
    return 1;
  } else {
    return 0;
  }
};

/**
 * Return the server name of the task.
 * @return {string}
 */
ZxOperation.prototype.getServerName = function() {
  return this.server;
};

/**
 * Return the operation name.
 * In the most case is useless.
 * @return {string}
 */
ZxOperation.prototype.getOpName = function() {
  return this.opName;
};

/**
 * Return the raw object returned from the server.
 * @return {{}}
 */
ZxOperation.prototype.getOpData = function() {
  return this.opData;
};

/**
 * Return the module name associated with the operation.
 * @return {string}
 */
ZxOperation.prototype.getModuleName = function() {
  return this.module;
};

/**
 * Return the task item of the task.
 * @return {ZaTaskItem|undefined}
 */
ZxOperation.prototype.getTaskItem = function() {
  return this._taskItem;
};

/**
 * Notify Operation updates.
 * @param {{}} diff
 */
ZxOperation.prototype.applyChanges = function(diff) {
  if (diff.opData && typeof diff.opData.state === 'string' &&
    this._taskItem && (typeof ZaTaskController !== 'undefined')) {
    // ZaZimbraAdmin.getInstance().getTaskController().removeTask(this._taskItem);
    this._taskItem.type = ZxOperation.getTaskItemTypeCode(diff.opData.state);
  }
  ZxUtils.log(diff, 'Updating Operation');
  ZxUtils.mergeObjects(this.opData, diff.opData);
  if (diff.opData && typeof diff.opData.state === 'string' &&
    this._taskItem && (typeof ZaTaskController !== 'undefined')) {
    // ZaZimbraAdmin.getInstance().getTaskController().addTask(this._taskItem);
  }

  var dialog;
  if (ZxOperation.getTaskItemTypeCode(this.opData.state) === 1) {
    dialog = ZxOperationDialog;
  } else if (ZxOperation.getTaskItemTypeCode(this.opData.state) === 2) {
    dialog = new ZxOperationDialog(ZaApp.getInstance().getAppCtxt().getShell(), this);
    dialog.setObject(this);
  }
  this._taskItem.viewForPopup = dialog;
};

/**
 * Get the id of the operation.
 * @return {string}
 */
ZxOperation.prototype.getID = function() {
  return this.uuid;
};

/**
 * Every task can have a type, the type define the panel on which the element must be show.
 * @param {string} state
 * @return {number}
 */
ZxOperation.getTaskItemTypeCode = function(state) {
  switch (state) {

    case 'Queued':
      return 2;

    case 'Started':
    default:
      return 1;
  }
};
