/*
 * ***** BEGIN LICENSE BLOCK *****
 * Copyright (C) 2011-2016 ZeXtras
 *
 * The contents of this file are subject to the ZeXtras EULA;
 * you may not use this file except in compliance with the EULA.
 * You may obtain a copy of the EULA at
 * http://www.zextras.com/zextras-eula.html
 * ***** END LICENSE BLOCK *****
 */

/**
 * Object that handle the operations queue.
 * It will automatically update queues operations informations.
 * @constructor
 */
function ZxOperationsManager() {
  this._operations = {};
  if (!ZeXtrasItem.isUserAdmin()) { return; }

  this._loadAction = new AjxTimedAction(this, this._update, {});
  this._emptyResponseNotified = false;
  this._networkErrorNotified = false;
  this._update();
}

/**
 * The update interval in minutes.
 * @type {number}
 */
ZxOperationsManager.UPDATE_INTERVALS = 0.50;

ZxOperationsManager.UPDATE_DELAY = 1000;

/**
 * Get the operation queue.
 * @return {ZxOperation[]}
 */
ZxOperationsManager.prototype.getAllOperations = function() {
  return this._operations;
};

/**
 * Return a @see ZxOperation dy id, if not present return undefined.
 * @param {string} id
 * @return {ZxOperation|undefined}
 */
ZxOperationsManager.prototype.getOperationByID = function(id) {
  if (this._operations.hasOwnProperty(id)) {
    return this._operations[id];
  }
};

/**
 * Get the Operation queue size.
 * @return {number}
 */
ZxOperationsManager.prototype.getQueueSize = function() {
  var count = 0,
    tmpName;
  for (tmpName in this._operations) {
    if (!this._operations.hasOwnProperty(tmpName)) { continue; }
    count = count + 1;
  }
  return count;
};

/**
 * Manage the sidebar updating displayed current operations.
 * @param {{}} newList
 */
ZxOperationsManager.prototype._updateSidebar = function(newList) {
  var diff = ZxUtils.objDiff(newList, this._operations, true),
    id,
    op,
    taskItem,
    oldState = 0,
    newState = 0;

  for (id in diff) {
    if (!diff.hasOwnProperty(id)) { continue; }

    op = this._operations[id];

    if (diff[id] === null) { // Task need to be removed
      if (op) {
        taskItem = op.getTaskItem();
        // if (taskItem && (typeof ZaTaskController !== 'undefined')) {
        //   ZaZimbraAdmin.getInstance().getTaskController().removeTask(taskItem);
        // }
        delete this._operations[id];
      }
    } else {
      oldState = 0;
      newState = 0;
      if (op) { // Task need to be updated
        // ZX-2841 Detect a change between states
        oldState = ZxOperation.getTaskItemTypeCode(op.getOpData().state);
        op.applyChanges(diff[id]);
        newState = ZxOperation.getTaskItemTypeCode(op.getOpData().state);
        if (newState !== oldState) {
          taskItem = this._operations[id].getTaskItem();
          if (taskItem && (typeof ZaTaskController !== 'undefined')) {
            taskItem.type = oldState;
            // ZaZimbraAdmin.getInstance().getTaskController().removeTask(taskItem);
            taskItem.type = newState;
            // ZaZimbraAdmin.getInstance().getTaskController().addTask(taskItem);
          }
        }

      } else { // Task to be added
        this._operations[id] = diff[id];
        taskItem = this._operations[id].getTaskItem();
        // if (taskItem && (typeof ZaTaskController !== 'undefined')) {
        //   ZaZimbraAdmin.getInstance().getTaskController().addTask(taskItem);
        // }
      }
    }
  }
};

/**
 * Refresh the operation list.
 * @param {boolean=} delayed if true delay the update, time is defined in ZxOperationsManager.UPDATE_DELAY
 * @return {Q.promise}
 */
ZxOperationsManager.prototype.requestUpdate = function(delayed) {
  delayed = delayed || false;

  var deferred = Q.defer(),
    updateCallback = AjxCallback.simpleClosure(
      function() {
        this._update().then(
          AjxCallback.simpleClosure(
            function(deferred, info) {
              deferred.resolve(info);
            },
            this,
            deferred
          ),
          AjxCallback.simpleClosure(
            function(deferred, error) {
              deferred.reject(error);
            },
            this,
            deferred
          )
        );
      },
      this,
      deferred
    );

  if (!ZeXtrasItem.isUserAdmin()) {
    deferred.resolve({});
    return deferred.promise;
  }

  if (delayed) {
    Q.delay(ZxOperationsManager.UPDATE_DELAY).then(updateCallback);
  } else {
    // Added a little delay to let the server to add an operation before to request the operation list.
    Q.delay(50).then(updateCallback);
  }

  return deferred.promise;
};

/**
 * Internal function triggered by AjxTimedAction every UPDATE_INTERVALS minutes.
 * This will update the internal list of the operations.
 * @return {Q.promise}
 */
ZxOperationsManager.prototype._update = function() {
  var deferred = Q.defer();

  if (this._scheduledActionID) {
    AjxTimedAction.cancelAction(this._scheduledActionID);
    delete this._scheduledActionID;
  }

  this._scheduledActionID = AjxTimedAction.scheduleAction(
    this._loadAction,
    ZxOperationsManager.UPDATE_INTERVALS * 60000
  );

  if (!ZeXtrasItem.isUserAdmin()) {
    deferred.resolve({});
    return deferred.promise;
  }

  ZxAdminRequest.sendRequest(
    ZxAdminRequest.TARGET_ALL,
    'ZxCore',
    'getAllOperations'
  ).then(
    AjxCallback.simpleClosure(
      function(deferred, resp) {
        this._emptyResponseNotified = false;
        this._networkErrorNotified = false;
        var server,
          list,
          newList = {},
          opName,
          tmpOp,
          n;

        for (server in resp) {
          if (!resp.hasOwnProperty(server)) { continue; }

          if (resp[server] instanceof ZxError) {
            // ZX-2846 If a response contains an error, ignore the response.
            continue;
          }

          list = resp[server].response.operationList;
          for (n = 0; n < list.length; n += 1) {
            tmpOp = new ZxOperation(list[n]);
            newList[tmpOp.getID()] = tmpOp;
          }
        }

        this._updateSidebar(newList);
        deferred.resolve(newList);
      },
      this,
      deferred
    ),
    AjxCallback.simpleClosure(
      this._catchLoadErrors,
      this,
      deferred
    )
  );

  return deferred.promise;
};

/**
 * Get if a FullScan or SmartScan is running on a server (if defined).
 * @param {string} serverName
 * @see ZxBackup
 * @return {boolean}
 */
ZxOperationsManager.prototype.isBackupScanRunning = function(serverName) {
  var id, checkserver = !!serverName;
  for (id in this._operations) {
    if (!this._operations.hasOwnProperty(id)) { continue; }

    if (this._operations[id].getModuleName() === 'ZxBackup' &&
        this._operations[id].getOpData().name === 'SmartScan') {

      if (checkserver) {
        if (this._operations[id].getServerName() === serverName) {
          return true;
        }
      } else {
        return true;
      }
    }
  }

  return false;
};

/**
 * Catch errors caused by loading methods.
 * @param {Q.defer} deferred
 * @param {ZxError|Error} err
 * @return {boolean}
 */
ZxOperationsManager.prototype._catchLoadErrors = function(deferred, err) {
  var mustShow = false;

  if (typeof err.getCode === 'function') {

    if (err.getCode() === ZxErrorCode.ZM_CSFE_EXCEPTION && !this._networkErrorNotified) {
      this._networkErrorNotified = true;
      mustShow = true;
    }

  }

  if (typeof err.getDetail === 'function' &&
      !!err.getDetail(ZxSOAPError.KEY_CODE)) {

    if (err.getDetail(ZxSOAPError.KEY_CODE) === ZmCsfeException.EMPTY_RESPONSE && !this._emptyResponseNotified) {
      this._emptyResponseNotified = true;
      this._networkErrorNotified = true;
      mustShow = true;
    }

  }

  if (mustShow) {
    ZxUtils.catchError(err);
  }

  this._scheduledActionID = AjxTimedAction.scheduleAction(
    this._loadAction,
    ZxOperationsManager.UPDATE_INTERVALS * 60000
  );
  ZxUtils.log(err, 'Error during refresh Operation list');
  deferred.reject(err);
  return true;
};
