/*
 * ***** BEGIN LICENSE BLOCK *****
 * Copyright (C) 2011-2016 ZeXtras
 *
 * The contents of this file are subject to the ZeXtras EULA;
 * you may not use this file except in compliance with the EULA.
 * You may obtain a copy of the EULA at
 * http://www.zextras.com/zextras-eula.html
 * ***** END LICENSE BLOCK *****
 */

if (typeof module !== 'undefined' && module.exports) {
  var require = patchRequire(require);
  ZxError = require('./ZxError');
  ZxErrorCode = require('./ZxErrorCode');
}

/**
 * ZxSOAP Error wrapper used to wrap errors.
 * @param {ZmCsfeException|AjxException|Error|ZxError|String=} error
 * @constructor
 */
function ZxSOAPError(error) {
  if (!error || typeof error === 'string') {
    ZxError.call(
      this,
      ZxErrorCode.GENERIC_ERROR,
      new Error(error)
    );

  } else if (error instanceof ZxError) {
    ZxError.call(
      this,
      error.getCode(),
      error
    );

  } else if (error instanceof ZmCsfeException) {
    var det;
    ZxError.call(
      this,
      ZxErrorCode.ZM_CSFE_EXCEPTION,
      error
    );
    for (det in error) {
      if (!error.hasOwnProperty(det)) { continue; }
      this._details[det] = error[det];
    }
    if (!error.hasOwnProperty('msg') || !error.msg) {
      this._details['msg'] = error.detail || '';
    }
  } else if (error instanceof AjxException) {
    var det;
    ZxError.call(
      this,
      ZxErrorCode.AJX_EXCEPTION,
      error
    );
    for (det in error) {
      if (!error.hasOwnProperty(det)) { continue; }
      this._details[det] = error[det];
    }
  } else if (error instanceof Error) {
    ZxError.call(
      this,
      ZxErrorCode.GENERIC_ERROR,
      error
    );

  } else {
    throw new TypeError('Invalid parameter [error].');
  }

  this._requestCause = null;
  this.name = 'ZxSOAPError';
}

ZxSOAPError.prototype = new ZxError();
ZxSOAPError.prototype.constructor = ZxSOAPError;

ZxSOAPError.TYPE_ERROR     = 'ZxSOAPError.TYPE_ERROR';
ZxSOAPError.TYPE_EXCEPTION = 'ZxSOAPError.TYPE_EXCEPTION';
ZxSOAPError.TYPE_HTTP      = 'ZxSOAPError.TYPE_HTTP';
ZxSOAPError.TYPE_NETWORK   = 'ZxSOAPError.TYPE_NETWORK';
ZxSOAPError.TYPE_GENERIC   = 'ZxSOAPError.TYPE_GENERIC';
ZxSOAPError.TYPE_ZXERROR   = 'ZxSOAPError.TYPE_ZXERROR';

ZxSOAPError.KEY_CODE = 'code';

/**
 * Get the error type.
 * @return {string}
 */
ZxSOAPError.prototype.getType = function() {
  return this._type;
};

/**
 * Get the error name.
 * @return {string}
 */
ZxSOAPError.prototype.getName = function() {
  return this.name;
};

/**
 * Get the original error object.
 * @return {{}}
 */
ZxSOAPError.prototype.getData = function() {
  return this._errorData;
};

/**
 * Attach a promise to the error object.
 * @param {Q.makePromise|Q.promise} promise
 */
ZxSOAPError.prototype.setPromise = function(promise) {
  if (!(promise instanceof Q.makePromise)) {
    throw new TypeError('[promise] must be a Q.makePromise');
  }
  this._promise = promise;
};

/**
 * Get if the error has a promise attached.
 * @return {boolean}
 */
ZxSOAPError.prototype.hasPromise = function() {
  return (!!this._promise);
};

/**
 * Get the promise attached to the error.
 * @return {Q.promise|undefined}
 */
ZxSOAPError.prototype.getPromise = function() {
  return this._promise;
};

/**
 * Get the error code. (provided by ZmCsfeException, AjxException)
 * @return {string}
 */
ZxSOAPError.prototype.getErrorCode = function() {
  return this._errorCode || '';
};

/**
 * Set the request object that have caused the error.
 * @param {ZxSOAPRequest} request
 */
ZxSOAPError.prototype.setRequest = function(request) {
  this._requestCause = request;
};

/**
 * Get the request object in JSON format if set, string otherwise.
 * @return {{}|String}
 */
ZxSOAPError.prototype.getReqObj = function() {
  if (this._requestCause && this._requestCause._requestObjValues) {
    return this._requestCause._requestObjValues;
  }
  return 'Not set';
};

if (typeof module !== 'undefined' && module.exports) {
  module.exports = ZxSOAPError;
}
