/*
 * ***** BEGIN LICENSE BLOCK *****
 * Copyright (C) 2011, 2012, 2013, 2014 ZeXtras
 *
 * The contents of this file are subject to the ZeXtras EULA;
 * you may not use this file except in compliance with the EULA.
 * You may obtain a copy of the EULA at
 * http://www.zextras.com/zextras-eula.html
 * ***** END LICENSE BLOCK *****
 */

/**
 *
 * @param {string} requestType (ZxSOAPRequest.CHAT|ZxSOAPRequest.ADMINISTRATION).
 * @constructor
 */
function ZxSOAPRequest(requestType) {
  this._appCtxt = null;
  this.id = 0;
  if (typeof appCtxt !== 'undefined') {
    this._appCtxt = appCtxt;
  } else if (typeof ZaApp !== 'undefined' && ZaApp.getInstance() !== null) {
    this._appCtxt = ZaApp.getInstance();
  } else {
    throw new Error('Unable to build ZeXtras Request Object');
  }

  var mUrn = '';
  if (requestType === ZxSOAPRequest.CHAT) {
    mUrn = 'urn:zimbraAccount';
  } else if (requestType === ZxSOAPRequest.ADMINISTRATION) {
    mUrn = 'urn:zimbraAdmin';
  } else {
    throw new TypeError('[requestType] must be a valid ZeXtras request type.');
  }
  this._requestType = requestType;
  this._deferred = Q.defer();
  this._delayedCount = 0;

  this._soapDoc_params = {};
  this._soapDoc_params.soapDoc = AjxSoapDoc.create(requestType, mUrn);
  this._soapDoc_params.busyMsg = 'ZeXtras Requesting...';
  this._soapDoc_params.asyncMode = true;
  this._soapDoc_params.callback = new AjxCallback(this, this._resultCallback);
  this._soapDoc_params.errorCallback = new AjxCallback(this, this._errorCallback);
  this._soapDoc_params.noBusyOverlay = true;

  this._requestObjValues = {};
}

ZxSOAPRequest.CHAT = 'ZxChatRequest';
ZxSOAPRequest.ADMINISTRATION = 'zextras';

ZxSOAPRequest.RPC_CACHE_INTERVAL_RETRY = 250;
ZxSOAPRequest.OUT_OF_RPC_CACHE_DELAY = 1500;
ZxSOAPRequest.RPC_MAX_CONCURRENT_REQUESTS = 10;

ZxSOAPRequest._TOT_REQS = 0;
ZxSOAPRequest._CONCURRENT_REQS = 0;
ZxSOAPRequest._TOT_DELAYS = 0;
ZxSOAPRequest._TOT_503_DELAYS = 0;
ZxSOAPRequest._TOT_502_ERRORS = 0;
ZxSOAPRequest._DONE = 0;
ZxSOAPRequest._ERR = 0;

/**
 * Set an object to attach to the request.
 * @param {String} key
 * @param {Object|String|Number|Date} value
 */
ZxSOAPRequest.prototype.set = function(key, value) {
  this._requestObjValues[key] = value;

  if (value instanceof Date) {
    this._soapDoc_params.soapDoc.set(key, value);

  } else if (value instanceof Object) {
    this._soapDoc_params.soapDoc.set(key, JSON.stringify(value));

  } else {
    if (typeof value === 'boolean') {
      this._soapDoc_params.soapDoc.set(key, value ? 'true' : 'false');
    } else if (typeof value === 'number') {
      this._soapDoc_params.soapDoc.set(key, value.toString());
    } else {
      this._soapDoc_params.soapDoc.set(key, value);
    }
  }
};

ZxSOAPRequest.prototype.setId = function(id) {
  this._requestObjValues.id = id;
  this.id = id;
};

ZxSOAPRequest.prototype.getRequestParameters = function() {
  return this._requestObjValues;
};

ZxSOAPRequest.prototype.getDeferred = function() {
  return this._deferred;
};

ZxSOAPRequest.prototype.getPromise = function() {
  return this._deferred.promise;
};

/**
 * The default callback analize the response to find exceptions.
 * If an exception is found, will be invoked the default error callback.
 * @param {XMLHttpRequest} response
 * @return {boolean|undefined}
 */
ZxSOAPRequest.prototype._resultCallback = function(response) {
  var resp, err1, err2;

  ZxSOAPRequest._CONCURRENT_REQS -= 1;

  try {
    if (response instanceof XMLHttpRequest) {
      resp = JSON.parse(response.responseText).Body.response;
    } else if (this._requestType === ZxSOAPRequest.ADMINISTRATION) {
      resp = response.getResponse().Body.response;
    } else {
      resp = response.getResponse().response;
    }

    if (resp.exception) {
      if (this._delayedCount > 0) {
        ZxSOAPRequest._TOT_503_DELAYS -= 1;
      }
      ZxSOAPRequest._ERR += 1;
      err1 = new ZxSOAPError(ZxError.fromResponse(resp));
      err1.setPromise(this._deferred.promise);
      err1.setRequest(this);
      this._deferred.reject(err1);
      return true;
    }

    if (resp.error) {
      if (this._delayedCount > 0) {
        ZxSOAPRequest._TOT_503_DELAYS -= 1;
      }
      ZxSOAPRequest._ERR += 1;
      err2 = new ZxSOAPError(ZxError.fromResponse(resp));
      err2.setPromise(this._deferred.promise);
      err2.setRequest(this);
      this._deferred.reject(err2);
      return true;
    }

  } catch (ex) {
    // Handle 503 Error (DoS Filter)
    if (ex.code && ex.detail && ex.code === ZmCsfeException.CSFE_SVC_ERROR &&
        (/503/.test(ex.detail))) {
      this._delayedCount += 1;
      if (this._delayedCount === 1) {
        ZxSOAPRequest._TOT_503_DELAYS += 1;
      }
      // If 503 error may be caused by DOSFilter, delay the request again.
      ZxUtils.log('DosFilter detected, delaying request...');
      if (typeof ZxDoSFilterWarningDialog === 'function') {
        var dosDialog = new ZxDoSFilterWarningDialog();
      }
      Q.delay(ZxSOAPRequest.OUT_OF_RPC_CACHE_DELAY).then(
        AjxCallback.simpleClosure(
          this.sendRequest,
          this
        )
      );
      return true;
    }
    if (this._delayedCount > 0) {
      ZxSOAPRequest._TOT_503_DELAYS -= 1;
    }
    ZxSOAPRequest._ERR += 1;
    err2 = new ZxSOAPError(ex);
    err2.setPromise(this._deferred.promise);
    err2.setRequest(this);
    this._deferred.reject(err2);
    return true;
  }

  ZxSOAPRequest._DONE += 1;
  if (this._delayedCount > 0) {
    ZxUtils.log(this._delayedCount, 'Resolving delayed (DOS Filter)');
    ZxSOAPRequest._TOT_503_DELAYS -= 1;
  }
  this._deferred.resolve(resp);
  return true;
};

/**
 * Default error callback.
 * @param {ZmCsfeException|Error} ex
 * @return {boolean}
 */
ZxSOAPRequest.prototype._errorCallback = function(ex) {
  var error,
    err,
    newEx;

  // Handle 502 Error (Proxy Error)
  if (ex.code && ex.detail && ex.code === ZmCsfeException.CSFE_SVC_ERROR && (/502/.test(ex.detail))) {
    ZxSOAPRequest._TOT_502_ERRORS += 1;
    error = new ZxError(ZxErrorCode.DETECTED_502);
    error.setDetail('detail', ex.detail);
    error.initCause(new ZxSOAPError(ex));
    this._deferred.reject(error);
    return true;
  }

  ZxSOAPRequest._ERR += 1;
  ZxSOAPRequest._CONCURRENT_REQS -= 1;
  if (this._delayedCount > 0) {
    ZxSOAPRequest._TOT_503_DELAYS -= 1;
  }

  try {
    err = new ZxSOAPError(ex);
    err.setPromise(this._deferred.promise);
    err.setRequest(this);
    this._deferred.reject(err);
  } catch (newError) {
    newEx = new ZxSOAPError(newError);
    newEx.setPromise(this._deferred.promise);
    newEx.setRequest(this);
    this._deferred.reject(newEx);
  }

  return true;
};

/**
 * Made the request synchronous.
 * By Default the requests should be asynchronous.
 */
ZxSOAPRequest.prototype.setSynchronous = function() {
  this._soapDoc_params.asyncMode = false;
};

/**
 * Do the request, if is not async return the result.
 * @param {Number=} timeout (optional)
 * @return {Q.promise|Object}
 */
ZxSOAPRequest.prototype.sendRequest = function(timeout) {
  if (ZmCsfeCommand.noAuth) {
    ZxUtils.log('Skipping request, user not authenticated');
    if (this._soapDoc_params.asyncMode) {
      return {};
    } else {
      return this._deferred.promise;
    }
  }

  if (timeout) {
    if (typeof timeout !== 'number') { throw '[timeout] must be a number'; }
    this._soapDoc_params.timeout = timeout;
  }

  var responseObj;
  try {
    if (ZxSOAPRequest._CONCURRENT_REQS < 0) {
      ZxSOAPRequest._CONCURRENT_REQS = 0;
    }

    if (this._requestType === ZxSOAPRequest.CHAT) {
      if (this._soapDoc_params.asyncMode) {
        this._appCtxt.getAppController().sendRequest(this._soapDoc_params);
      } else {
        return this._appCtxt.getAppController().sendRequest(this._soapDoc_params, this._soapDoc_params).response;
      }
    }
    if (this._requestType === ZxSOAPRequest.ADMINISTRATION) {
      this._soapDoc_params.controller = this._appCtxt.getCurrentController();

      if (this._soapDoc_params.asyncMode) {

        if (ZxSOAPRequest._CONCURRENT_REQS < ZxSOAPRequest.RPC_MAX_CONCURRENT_REQUESTS) {
          ZxSOAPRequest._TOT_REQS += 1;
          ZxSOAPRequest._CONCURRENT_REQS += 1;
          ZaRequestMgr.invoke(this._soapDoc_params, this._soapDoc_params);
        } else {
          ZxSOAPRequest._TOT_DELAYS += 1;
          Q.delay(ZxSOAPRequest.RPC_CACHE_INTERVAL_RETRY).then(
            AjxCallback.simpleClosure(
              this.sendRequest,
              this
            )
          );
        }
      } else {
        responseObj = ZaRequestMgr.invoke(this._soapDoc_params, this._soapDoc_params);
        return JSON.parse(responseObj.Body.response.content);
      }
    }
  } catch (err) {
    var error = new ZxSOAPError(err);
    error.setPromise(this._deferred.promise);
    error.setRequest(this);
    this._deferred.reject(error);
  }

  return this._deferred.promise;
};
