/*
 * ***** BEGIN LICENSE BLOCK *****
 * Copyright (C) 2011, 2012, 2013, 2014 ZeXtras
 *
 * The contents of this file are subject to the ZeXtras EULA;
 * you may not use this file except in compliance with the EULA.
 * You may obtain a copy of the EULA at
 * http://www.zextras.com/zextras-eula.html
 * ***** END LICENSE BLOCK *****
*/

if (typeof module !== 'undefined' && module.exports) {
  window = {};
  var require = patchRequire(require);
  printStackTrace = require('./printStackTrace.js');
  XRegExp = require('./xregexp.js');
  AjxUtil = require('../../../tests/js/moks/AjxUtil.js');
}

(function(context) {
  /**
   * Object containig utility functions used by ZeXtras
   */
  var ZxUtils = function() {};
  ZxUtils.version = 23;

  if (typeof context.ZxUtils === 'function') {
    if (context.ZxUtils.version && context.ZxUtils.version >= ZxUtils.version) {
      context.ZxUtils.log('ZxUtils already loaded, no update needed.');
      return;
    }
  }

  ZxUtils._isZxDevEnabled = false;
  ZxUtils.DEFAULT_REPORT_URL = '//update.zextras.com/feedback';
  ZxUtils.MSEC_PER_DAY = 86400000;

  /**
   * Get if we are currently on user web interface.
   * @return {boolean}
   */
  ZxUtils.onUserClient = function() {
    return (typeof appCtxt === 'object');
  };

  /**
   * Get if we are currently on administration web interface.
   * @return {boolean}
   */
  ZxUtils.onAdminClient = function() {
    return (typeof ZaAppCtxt === 'function' && !ZxUtils.onUserClient());
  };

  /**
   * @return {string}
   */
  ZxUtils.UTCDateString = function(d, def) {
    var month_abbrv = ZxUtils.MONTHS_ARRAY;

    if (d.valueOf() === 0) {
      if (!def) {
        def = 'n/a';
      }
      return def;
    }

    function pad(n) {
      return n < 10 ? '0' + n : n;
    }

    return pad(d.getUTCDate()) + ' ' +
      month_abbrv[d.getUTCMonth()] + ' ' +
      d.getUTCFullYear() + ' ' +
      pad(d.getUTCHours()) + ':' +
      pad(d.getUTCMinutes()) + ' UTC';
  };

  /**
   * Return the number of given days, into milliseconds.
   * @param {number} days
   * @return {number}
   */
  ZxUtils.DaysToMillis = function(days) {
    if (days === 0) {
      days = 1;
    }
    return days * 24 * 60 * 60 * 1000;
  };

  /**
   * Convert the HSM type numeber to a human-readable type.
   * @param {number} hsmTypes
   * @return {string}
   */
  ZxUtils.itemTypeToString = function(hsmTypes) {
    var n, type, type_string = '',
      types = {
        1: 'folder',
        2: 'searchfolder',
        3: 'tag',
        4: 'conversation',
        5: 'message',
        6: 'contact',
        7: 'invite',
        8: 'document',
        9: 'note',
        10: 'flag',
        11: 'appointment',
        12: 'virtual_conversation',
        13: 'mountpoint',
        14: 'wiki',
        15: 'task',
        16: 'chat',
        17: 'comment',
        18: 'link'
      };

    hsmTypes.sort();

    for (n in hsmTypes) {
      type = hsmTypes[n];
      type_string += types[type] + ',';
    }

    return type_string.replace(/,\s?$/, '');
  };

  /**
   * Transform bytes to a human-readable format.
   * @return {string}
   * @param {number} precision
   * @param {number} bytes
   */
  ZxUtils.bytesToSize = function(bytes, precision) {
    var kilobyte = 1024,
      megabyte = kilobyte * 1024,
      gigabyte = megabyte * 1024,
      terabyte = gigabyte * 1024;

    if ((bytes >= 0) && (bytes < kilobyte)) {
      return bytes + ' B';
    } else if ((bytes >= kilobyte) && (bytes < megabyte)) {
      return (bytes / kilobyte).toFixed(precision) + ' KB';
    } else if ((bytes >= megabyte) && (bytes < gigabyte)) {
      return (bytes / megabyte).toFixed(precision) + ' MB';
    } else if ((bytes >= gigabyte) && (bytes < terabyte)) {
      return (bytes / gigabyte).toFixed(precision) + ' GB';
    } else if (bytes >= terabyte) {
      return (bytes / terabyte).toFixed(precision) + ' TB';
    } else {
      return bytes;
    }
  };

  /**
   * Clone an object.
   * @param {{}} obj Object to clone.
   * @return {{}} A clone of the object.
   */
  ZxUtils.cloneObj = function(obj) {
    var copy;
    // Handle the 3 simple types, and null or undefined
    if (null === obj || 'object' !== (typeof obj)) { return obj; }

    // Handle Date
    if (obj instanceof Date) {
      copy = new Date();
      copy.setTime(obj.getTime());
      return copy;
    }

    // Handle Array
    if (obj instanceof Array) {
      copy = [];
      var i, len;
      for (i = 0, len = obj.length; i < len; i += 1) {
        copy[i] = ZxUtils.cloneObj(obj[i]);
      }
      return copy;
    }

    // Handle Object
    if (obj instanceof Object) {
      copy = {};
      var attr;
      for (attr in obj) {
        if (obj.hasOwnProperty(attr)) {
          copy[attr] = ZxUtils.cloneObj(obj[attr]);
        }
      }
      return copy;
    }

    throw new Error("Unable to copy obj! Its type isn't supported.");
  };

  /**
   * Get if a server has a Zimbra instance up and running.
   * @param {string} serverId
   * @return {boolean}
   */
  ZxUtils.isZimbraRunning = function(serverId) {
    var soapDoc = AjxSoapDoc.create('BatchRequest', 'urn:zimbra'),
      command,
      params;
    soapDoc.setMethodAttribute('onerror', 'continue');

    command = new ZmCsfeCommand();
    soapDoc.set('GetVersionInfoRequest', null, null, ZaZimbraAdmin.URN);
    params = {soapDoc: soapDoc, noAuthToken: true, targetServer: serverId};

    try {
      command.invoke(params);
    } catch (ex) {
      return false;
    }
    return true;
  };

  /**
   * Get the server ID by name.
   * @param {string} serverName
   * @return {string|undefined}
   */
  ZxUtils.getServerIDByName = function(serverName) {
    var servArr = ZaServer.getAllMBSs().getArray(),
      i = 0;

    for (i = 0; i < servArr.length; i += 1) {
      if (servArr[i].name === serverName) {
        return servArr[i].id;
      }
    }
  };

  /**
   * Get all parameters defined with "Get" method, into the url of the page.
   * @return {{}}
   */
  ZxUtils._getAllGetParameters = function() {
    var i, tmparr, prmstr = window.location.search.substr(1),
      prmarr = prmstr.split('&'),
      params = {};

    for (i = 0; i < prmarr.length; i += 1) {
      tmparr = prmarr[i].split('=');
      params[tmparr[0]] = tmparr[1];
    }

    return params;
  };

  /**
   * Get value of given paramenter, defined with "Get" method.
   * Can be used: Client, Admin
   * @param {String} param Paramenter name
   * @return {*}
   */
  ZxUtils.getGetParameter = function(param) {
    var params = ZxUtils._getAllGetParameters();
    if (!params.hasOwnProperty(param)) {
      return false;
    } else {
      return params[param];
    }
  };

  /**
   * Get if the client is running with 'zxdev=1'
   * (ZeXtras development mode)
   * @return {boolean}
   */
  ZxUtils.isZxDev = function() {
    return (ZxUtils.getGetParameter('zxdev') === '1' || ZxUtils._isZxDevEnabled);
  };

  /**
   * Enable the ZxDev mode, this mode allow to show the log into the debugger of the browser.
   */
  ZxUtils.enableZxDev = function() {
    ZxUtils._isZxDevEnabled = true;
  };

  /**
   * Disable the ZxDev mode, this mode allow to show the log into the debugger of the browser.
   */
  ZxUtils.disableZxDev = function() {
    ZxUtils._isZxDevEnabled = false;
  };

  ZxUtils.LOG_DEBUG = 'log'; /* 'debug' <- Deprecated */
  ZxUtils.LOG_ERROR = 'error';
  ZxUtils.LOG_INFO = 'info';
  ZxUtils.LOG_LOG = 'log';
  ZxUtils.LOG_WARNING = 'warn';

  ZxUtils._allowedLogs = ['debug', 'error', 'info', 'log', 'warn'];

  /**
   * Log a message or object.
   * Can be used: Client, Admin
   * @param {{}|[]|string|ZxError|number} obj will be shown into the console of the browser,
   *               If is an array show all contained items.
   * @param {string=} [title] title .
   * @param {string=} [priority] priority Use ZxUtils.LOG_* variable (default = ZxUtils.LOG_LOG).
   * @param {boolean=} [prettyPrint] prettyPrint If TRUE, the object will be converted to a string.
   * @constructor
   */
  ZxUtils.log = function(obj, title, priority, prettyPrint) {
    if (typeof console === 'undefined') {
      return "I hate IE";
    }
    if ((typeof appDevMode !== 'undefined' && appDevMode === true) || ZxUtils.isZxDev()) {
      try {
        var indexOf = function(array, object, strict) {
            if (array) {
              for (var i = 0; i < array.length; i++) {
                var item = array[i];
              }
              if ((strict && item === object) || (!strict && item == object)) {
                return i;
              }
            }
          },
          stack = printStackTrace(),
          sLine = '',
          jsFile = '',
          jsLine = '0',
          date = new Date();

        title = title || false;

        if (indexOf(ZxUtils._allowedLogs, title) < -1) {
          priority = title;
          title = false;
        }

        priority = priority || ZxUtils.LOG_LOG;
        if (indexOf(ZxUtils._allowedLogs, priority) === -1) {
          priority = ZxUtils.LOG_LOG;
        }
        if (!console.hasOwnProperty(priority)) {
          priority = ZxUtils.LOG_LOG;
        }

        if (typeof console !== 'undefined' && typeof console !== 'string') {
          var sLine = stack[4],
            regex = XRegExp(
                '@[^ ]+\\/' +
                '(?<FileName>[^ ]+)' +
                '\\?[^ :]+:' +
                '(?<Line>\\d+)',
              'x'),
            tmp = XRegExp.exec(sLine, regex) || [];

          jsFile = tmp.FileName || '';
          jsLine = tmp.Line || '';

          var groupTitle = ['[', date.getDate(), '/', date.getMonth(), '/', date.getFullYear(), ' ',
            date.getHours(), ':', date.getMinutes(), '.', date.getMilliseconds(), '] ',
            jsFile, ':', jsLine, (title ? ' - ' : ''), (title || '')].join('');
          if (typeof console.group !== 'undefined') {
            console.group(groupTitle);
          }
          if (Object.prototype.toString.call(obj) === '[object Array]') {
            var i;
            for (i = 0; i < obj.length; i += 1) {
              if (prettyPrint) {
                console[priority](JSON.stringify(obj[i], null, 2));
              } else {
                console[priority](obj[i]);
              }
            }
          } else {
            if (prettyPrint) {
              console[priority](JSON.stringify(obj, null, 2));
            } else {
              console[priority](obj);
            }
          }
          if (typeof console.groupEnd !== 'undefined') {
            console.groupEnd();
          }
        } else {
          if (title) {
            console.log(title);
          }
          console.log(obj);
        }

      } catch (e) {
        //alert(obj);    Do not use a fallback mode
      }
    }
  };

  /**
   * Compare version in format major.minor.micro[.patch].
   * Can return:
   *  * -1 if a > b
   *  * 0 if a = b
   *  * 1 if a < b
   *
   * @param {string} a
   * @param {string} b
   * @return {number}
   */
  ZxUtils.compareVersions = function(a, b) {
    if (typeof a === 'string' && typeof b === 'string') {
      // Clean the version string
      a = a.split('_')[0];
      b = b.split('_')[0];
      var arrA = a.split('.'),
        arrB = b.split('.'),
        numA,
        numB,
        i;

      for (i = 0; i < arrA.length && i < arrB.length; i += 1) {
        if (i > 0) {
          arrA[i] = '0.' + arrA[i];
          arrB[i] = '0.' + arrB[i];
        }
        numA = Number(arrA[i]);
        numB = Number(arrB[i]);
        if (numA > numB) {
          return -1;
        }
        if (numA < numB) {
          return 1;
        }
      }
      if (arrA.length > arrB.length) {
        return -1;
      }
      if (arrA.length < arrB.length) {
        return 1;
      }
      return 0;
    }
  };


  /**
   * Compare version in format major.minor.micro[.patch].
   * Can return:
   *  * false if a[ZxChat.KEY_CORE_VERSION_MINIMUM] > b[ZxChat.KEY_CORE_VERSION_CURRENT]
   *  * true if a[ZxChat.KEY_CORE_VERSION_MINIMUM] <= b[ZxChat.KEY_CORE_VERSION_CURRENT]
   *
   * @param {string} a
   * @return {boolean}
   */
  ZxUtils.isCurrentCoreVersion_Oldest_MinimumCoreVersion = function(a) {
    if (ZxUtils.compareVersions(a[ZxChat.KEY_CORE_VERSION_MINIMUM], a[ZxChat.KEY_CORE_VERSION_CURRENT]) < 0) {
      return true;
    } else {
      return false;
    }
  };

  /**
   * Return 0 of versions are equals, -1 if requested<current, 1 if current<requested
   * @param {string} requestedVersion must be like "8.0.0".
   * @return {number}
   */
  ZxUtils.CompareZimbraVersion = function(requestedVersion) {
    var currentVersion = '0.0.0';

    if (typeof ZaServerVersionInfo !== 'undefined') {
      currentVersion = ZaServerVersionInfo.version;
    } else {
      currentVersion = appCtxt.get(ZmSetting.CLIENT_VERSION);
    }

    return ZxUtils.compareVersions(requestedVersion, currentVersion);
  };

  /**
   * Return if the current Zimbra version is >= 8
   * @return {boolean}
   */
  ZxUtils.isZ8Up = function() {
    return ZxUtils.CompareZimbraVersion('8.0.0') > -1;
  };

  /**
   * Return if the current Zimbra version is >= 7
   * @return {boolean}
   */
  ZxUtils.isZ7Up = function() {
    return ZxUtils.CompareZimbraVersion('7.0.0') > -1;
  };

  /**
   * Return if the current Zimbra version is === 6
   * @return {boolean}
   */
  ZxUtils.isZ6 = function() {
    return ZxUtils.CompareZimbraVersion('6.0.0') > -1 && !ZxUtils.isZ7Up();
  };

  /**
   * Sanitize a tring, escaping special HTML entities.
   * Can be used: Client, Admin
   * @param {String} string which need sanitization.
   * @return {String}
   */
  ZxUtils.HtmlEscape = function(string) {
    return String(string)
      .replace(/&/g, '&amp;')
      .replace(/"/g, '&quot;')
      .replace(/'/g, '&#39;')
      .replace(/</g, '&lt;')
      .replace(/>/g, '&gt;')
      .replace(/\r/g, '');
  //    .replace(/\n/g, '<br>');
  };

  /**
   * Unescape HTML entities.
   * Can be used: Client, Admin
   * @param {String} str
   * @return {String}
   */
  ZxUtils.HtmlUnEscape = function(str) {
    return String(str)
      .replace(/&amp;/g, '&')
      .replace(/&quot;/g, '"')
      .replace(/&#39;/g, '\'')
      .replace(/&lt;/g, '<')
      .replace(/&gt;/g, '>')
      .replace(/<br>/g, '\n');
  };

  /**
   * Axtract an email address from the input text.
   * Can be used: Client, Admin
   * @param {string} text  Input text
   * @return {String}
   */
  ZxUtils.ExtractEmail = function(text) {
    return text.replace(/([^ ;,:]*@[^ ;,:]*).*/, '$1');
  };

  ZxUtils.bindEvent = function(el, eventName, eventHandler) {
    if (el.addEventListener) {
      el.addEventListener(eventName, eventHandler, false);
    } else if (el.attachEvent) {
      el.attachEvent('on' + eventName, eventHandler);
    }
  };

  /**
   * Check if a string end with another string.
   * Can be used: Client, Admin
   * @param {String} str
   * @param {String} suffix
   * @return {boolean}
   */
  ZxUtils.endsWith = function(str, suffix) {
    return str.indexOf(suffix, str.length - suffix.length) !== -1;
  };

  /**
   * Get current selection position inside an HTMLElement.
   * Can be used: Client, Admin
   * @param {HTMLElement} el
   * @return {number}
   */
  ZxUtils.getCaret = function(el) {
    var caretOffset = 0,
      doc = el.ownerDocument || el.document,
      win = doc.defaultView || doc.parentWindow,
      sel,
      range,
      preCaretRange,
      textRange,
      preCaretTextRange;
    if (el.selectionStart || el.selectionStart === 0) {
      // Firefox Support
      caretOffset = el.selectionStart - 1;
    } else if (typeof win.getSelection !== 'undefined') {
      range = win.getSelection().getRangeAt(0);
      preCaretRange = range.cloneRange();
      preCaretRange.selectNodeContents(el);
      preCaretRange.setEnd(range.endContainer, range.endOffset);
      caretOffset = preCaretRange.toString().length;
    } else if ((sel = doc.selection) && sel.type !== 'Control') {
      textRange = sel.createRange();
      preCaretTextRange = doc.body.createTextRange();
      preCaretTextRange.moveToElementText(el);
      preCaretTextRange.setEndPoint('EndToEnd', textRange);
      caretOffset = preCaretTextRange.text.length;
    }
    return caretOffset;
  };

  /**
   * Get an error dialog, modified to send error to the ZeXtras team.
   * Can be used: Client, Admin
   * @param {ZxError=} error
   * @return {ZmErrorDialog|ZaErrorDialog|undefined}
   */
  ZxUtils.getErrorDialog = function(error) {
    var errorDialog;

    if (ZxUtils.onUserClient()) {
      AjxDispatcher.require('Startup2');
      errorDialog = new ZmErrorDialog(appCtxt.getShell(), ZmMsg);
      errorDialog.unregisterCallback(ZmErrorDialog.REPORT_BUTTON);
      errorDialog.registerCallback(ZmErrorDialog.REPORT_BUTTON, ZxUtils._ErrorReportCallback, errorDialog, [error]);
    }

    if (ZxUtils.onAdminClient()) {
      errorDialog = new ZaErrorDialog(ZaApp.getInstance().getAppCtxt().getShell());

      var detailBtn,
        mAppCtxt = ZaApp.getInstance().getAppCtxt(),
        sendButton = new DwtDialog_ButtonDescriptor(1, 'Send Report', DwtDialog.ALIGN_LEFT,
          new AjxCallback(errorDialog, ZxUtils._ErrorReportCallback, [error])),
        detailButton = new DwtDialog_ButtonDescriptor(ZaErrorDialog.DETAIL_BUTTON, AjxMsg.detail, DwtDialog.ALIGN_LEFT);

      DwtMessageDialog.call(errorDialog, mAppCtxt.getShell(), null, null, [sendButton, detailButton]);
      errorDialog._detailCell = document.getElementById(errorDialog._detailCellId);
      detailBtn = errorDialog._button[ZaErrorDialog.DETAIL_BUTTON];
      detailBtn.setImage('SelectPullDownArrow');
      errorDialog.registerCallback(ZaErrorDialog.DETAIL_BUTTON, errorDialog._showDetail, errorDialog);
      errorDialog._setAllowSelection();
    }

    if (!errorDialog) {
      ZxUtils.log('Unable to create error dialog');
    }

    return errorDialog;
  };

  /**
   * Handle an error displaying it in human-readable format.
   * @param {ZxError=} ex
   * @param {boolean} allowReport
   * @return {boolean}
   */
  ZxUtils.popupErrorDialog = function(ex, allowReport) {
    var newException;

    if (!(ex instanceof ZxError)) {
      if (ex instanceof ZmCsfeException) {
        newException = new ZxError(ZxErrorCode.ZM_CSFE_EXCEPTION, ex);
      } else {
        newException = new ZxError(ZxErrorCode.UNKNOWN_JS_EXCEPTION, ex);
      }
    } else {
      newException = ex;
    }

    var ed = ZxUtils.getErrorDialog(newException),
      errorMessage,
      errorTitle = 'ZeXtras Request Error.';

    errorMessage = [
      'Client: ' + ((ZxUtils.onUserClient()) ? 'User Interface' : 'Administration'), '<br>'
    ];

    if (ex instanceof ZmCsfeException) {
      if (ex.code === ZmCsfeException.CSFE_SVC_ERROR) {
        ZxUtils.log('Ignoring 503 Error');
        return true;
      }

      if (ex.code === ZmCsfeException.SVC_UNKNOWN_DOCUMENT) {
        ZxUtils.log('Ignoring UNKNOWN_DOCUMENT Error');
        return true;
      }
    }

    errorTitle = newException.getMessage();
    errorMessage.push(newException.toString());

    ed.setMessage(
      errorTitle,
      errorMessage,
      DwtMessageDialog.CRITICAL_STYLE,
      ZxUtils.getTitleWithIcon('ZeXtras', 'ZeXtras Error')
    );
    if (!allowReport) {
      if (typeof ZmErrorDialog !== 'undefined' && ed instanceof ZmErrorDialog) {
        ed.setButtonVisible(ZmErrorDialog.REPORT_BUTTON, false);
      }

      if (typeof ZaErrorDialog !== 'undefined' && ed instanceof ZaErrorDialog) {
        ed.setButtonVisible(DwtDialog.CANCEL_BUTTON, false);
      }
    }
    ed.popup();
  };

  /**
   * Get if an element is child of another.
   * Can be used: Client, Admin
   * @param child
   * @param parent
   * @return {boolean}
   */
  ZxUtils.isChildOf = function(child, parent) {
    var node = child.parentNode;
    while (node !== null) {
      if (node === parent) {
        return true;
      }
      node = node.parentNode;
    }
    return false;
  };

  /**
   * Get if there is Zimbra Im App
   * Can be used: Client
   * @return {boolean}
   */
  ZxUtils.thereIsZIm = function() {
    if (!ZxUtils.onUserClient()) { return false; }
    return appCtxt.getApp(ZmApp.IM);
  };

  /* Hacks DWT for Zimbra 7 compatibility */


  /**
   * Callback used to overwrite the error reporting.
   */
  ZxUtils._ErrorReportCallback = function(error) {
    if (typeof appCtxt !== 'undefined') {
      var soapDoc = AjxSoapDoc.create('ZxErrorReport', 'urn:zimbraAccount');
      soapDoc.set('ZxError', JSON.stringify(error.toJSON()));
      soapDoc.set('environment', window.navigator.userAgent);
      appCtxt.getAppController().sendRequest({soapDoc: soapDoc});
      appCtxt.setStatusMsg(
        (typeof ZmMsg !== 'undefined') ? ZmMsg.errorReportSent : 'Error report sent to ZeXtras Team'
      );
      this.popdown();
    } else {
      ZxAdminRequest.sendRequest(
        ZxAdminRequest.TARGET_LOCAL,
        'ZxCore',
        'doReport',
        {
          ZxError: error.toJSON(),
          environment: window.navigator.userAgent
        }
      ).then(
        AjxCallback.simpleClosure(
          function(resp) {
            this.popdown();
            ZxUtils.log(resp, 'Report sent');
          },
          this
        ),
        ZxUtils.catchError
      );
    }

  };

  /**
   * Find all urls in the given text and wrap them in an 'a' HTML element.
   * Can be used: Client, Admin
   * @param {String} text
   * @return {String}
   */
  ZxUtils.FindURLs = function(text) {
    var tmpArray = text.split(' '), i = 0;
    try {
      for (i = 0; i < tmpArray.length; i += 1) {
        if (AjxUtil.FULL_URL_RE.test(tmpArray[i])) {
          tmpArray[i] = ['<a href="', tmpArray[i], '" target="_blank">', tmpArray[i], '</a>'].join('');
        }
      }
    } catch (err) {
      /* If an error occurred, return the text given in input. */
      return text;
    }

    return tmpArray.join(' ');
  };

  /**
   * Get a hour formatted using local rules
   * 15:50 can be: 15.50 or 3.50pm
   * @param {Date} date
   * @return {String} Formatted by local rules
   * @constructor
   */
  ZxUtils.localizeHour = function(date) {
    var hours = date.getHours(),
      minutes = date.getMinutes(),
      dateStr = [],
      tmpHour;

    if (AjxDateUtil.isLocale24Hour()) {
      dateStr.push(hours < 10 ? '0' : '');
      dateStr.push(hours);
      dateStr.push(':');
      dateStr.push(minutes < 10 ? '0' : '');
      dateStr.push(minutes);
    } else {
      tmpHour = (hours < 13 ? hours : (hours - 12));
      tmpHour = (tmpHour < 1 ? 12 : tmpHour);
      dateStr.push(tmpHour);
      dateStr.push(':');
      dateStr.push(minutes < 10 ? '0' : '');
      dateStr.push(minutes);
      dateStr.push(hours < 12 ? ' AM' : ' PM');
    }

    return dateStr.join('');
  };

  /**
   * Force to load the CSS for the given zimlet
   * Can be used: Client, Admin
   * @param {String} zimletPkgName
   * @param {String} fileName
   */
  ZxUtils.loadCSS = function(zimletPkgName, fileName) {
    var file = ['/service/zimlet', zimletPkgName, 'css', fileName].join('/'),
      ss = document.styleSheets,
      head,
      style,
      i,
      max;
    for (i = 0, max = ss.length; i < max; i += 1) {
      if (ss[i].href === file) {
        ZxUtils.log(file + ' Already Loaded');
        return;
      }
    }
    head = document.getElementsByTagName('head')[0];
    style = document.createElement('link');
    style.type = 'text/css';
    style.rel = 'stylesheet';
    style.href = file;
    head.appendChild(style);
  };

  /**
   * Format a date into a short human-readable string in the format:
   * YYYY/MM/DD - HH:mm
   * @param {Date} date
   * @return {string}
   */
  ZxUtils.formatDateShort = function(date) {
    var formattedDate, day, month, year, hours, minutes;

    day = date.getDate();
    if (day < 10) {
      day = '0' + day;
    }

    month = date.getMonth() + 1;
    if (month < 10) {
      month = '0' + month;
    }

    year = date.getFullYear();

    hours = date.getHours();
    if (hours < 10) {
      hours = '0' + hours;
    }

    minutes = date.getMinutes();
    if (minutes < 10) {
      minutes = '0' + minutes;
    }

    formattedDate = [year, '/', month, '/', day, '-', hours, ':', minutes].join('');
    return formattedDate;
  };

  /**
   * Generate a random password with given complexity parameters
   * @param {number=12} length
   * @param {number=2} howManyNumbers
   * @param {number=2} howManyCaps
   * @return {String}
   */
  ZxUtils.generateRandomPassword = function(length, howManyNumbers, howManyCaps) {
    if (length == null) {
      length = 12;
    }
    if (howManyNumbers == null) {
      howManyNumbers = 2;
    }
    if (howManyCaps == null) {
      howManyCaps = 2;
    }
    var realLength = length,
      getRandomInt = function(min, max) {
        return Math.floor(Math.random() * (max - min)) + min;
      },
      password,
      position,
      i,

      remainingNumbers,
      remainingCapitals;

    if (howManyNumbers + howManyCaps > realLength) {
      realLength = howManyNumbers + howManyCaps;
    }
    password = [];
    i = 0;
    // Generate the string
    for (i = 0; i < realLength; i++) {
      password[i] = String.fromCharCode(getRandomInt(97, 97+25));
    }
    // Replace the numbers
    remainingNumbers = Math.min(realLength, howManyNumbers);
    if (remainingNumbers < 0) {
      remainingNumbers = 0;
    }
    while (remainingNumbers > 0) {
      position = getRandomInt(0, realLength);
      if (password[position] >= 'a' && password[position] <= 'z') {
        password[position] = getRandomInt(0, 9);
        remainingNumbers--;
      }
    }
    // Replace letters with some
    remainingCapitals = Math.min(realLength - howManyNumbers, howManyCaps);
    if (remainingCapitals < 0) {
      remainingCapitals = 0;
    }
    while (remainingCapitals > 0) {
      position = getRandomInt(0, realLength);
      if ((!(password[position] >= 0 && password[position] <= 9)) &&
        (password[position] >= 'a' && password[position] <= 'z'))
      {
        password[position] = password[position].toUpperCase();
        remainingCapitals--;
      }
    }

    return password.join('');
  };

  ZxUtils.replaceAll = function(string, find, replacement) {
    if (find !== replacement) {
      while (string.indexOf(find) > -1) {
        string = string.replace(find, replacement);
      }
    }
    return string;
  };

  /**
   * Return a translated string by label.
   * If is provided  substitutions array, this method provide to format the message.
   * @param {String} messageLabel
   * @param {string[]} [substitutions] substitutions
   * @return {String}
   */
  ZxUtils.getMessage = function(messageLabel, substitutions) {
    var msg = messageLabel,
      translation = messageLabel;
    substitutions = substitutions || [];
    try {
      if (typeof com_zextras_zextras !== 'undefined' && com_zextras_zextras[messageLabel]) {
        translation = com_zextras_zextras[messageLabel];
        msg = AjxMessageFormat.format(translation, substitutions);
      }
      if (typeof com_zextras_zxmig !== 'undefined' && com_zextras_zxmig[messageLabel]) {
        translation = com_zextras_zxmig[messageLabel];
        msg = AjxMessageFormat.format(translation, substitutions);
      }
      if (typeof com_zextras_chat !== 'undefined' && com_zextras_chat[messageLabel]) {
        translation = com_zextras_chat[messageLabel];
        msg = AjxMessageFormat.format(translation, substitutions);
      }
      if (typeof com_zextras_chat_testing !== 'undefined' && com_zextras_chat_testing[messageLabel]) {
        translation = com_zextras_chat_testing[messageLabel];
        msg = AjxMessageFormat.format(translation, substitutions);
      }
    } catch (err) {
      return translation;
    }

    return msg;
  };

  /**
   * Validate if the input string is a (formally) valid email address.
   * @param {string} email
   * @return {boolean}
   */
  ZxUtils.validateEmail = function(email) {
    var re = /^(([^<>()[\]\\.,;:\s@\"]+(\.[^<>()[\]\\.,;:\s@\"]+)*)|(\".+\"))@((\[[0-9]{1,3}\.[0-9]{1,3}\.[0-9]{1,3}\.[0-9]{1,3}\])|(([a-zA-Z\-0-9]+\.)+[a-zA-Z]{2,}))$/;
    return re.test(email);
  };

  /**
   * Recursively merge properties of two objects.
   * @param {{}} obj1
   * @param {{}} obj2
   * @return {{}}
   */
  ZxUtils.mergeObjects = function(obj1, obj2) {
    obj1 = obj1 || {};
    obj2 = obj2 || {};
    var p;
    for (p in obj2) {
      if (!obj2.hasOwnProperty(p)) { continue; }
      try {
        if (typeof obj2[p] !== 'undefined' &&
            typeof obj2[p].constructor !== 'undefined' &&
            obj2[p].constructor === Object) {
          obj1[p] = ZxUtils.mergeObjects(obj1[p], obj2[p]);
        } else {
          obj1[p] = obj2[p];
        }
      } catch (e) {
        obj1[p] = obj2[p];
      }
    }
    return obj1;
  };

  /**
   * Create and return an object composed by the ref path
   * with the specified value.
   * @param {string} ref
   * @param {*} val
   * @return {{}}
   */
  ZxUtils.createObjectFromRef = function(ref, val) {
    var i = 0,
      arr = ref.split('/'),
      obj = {},
      pointer = obj;

    for (i = 0; i < arr.length; i += 1) {
      if (i === (arr.length - 1)) {
        pointer[arr[i]] = val;
      } else {
        pointer[arr[i]] = {};
        pointer = pointer[arr[i]];
      }
    }
    return obj;
  };

  /**
   * Return if an object is empty.
   * @param {*} obj
   * @return {boolean}
   */
  ZxUtils.isObjectEmpty = function(obj) {
    if (typeof obj !== 'object') {
      return false;
    }
    for (var name in obj) {
      if (!obj.hasOwnProperty(name)) {continue;}
      return false;
    }
    return true;
  };

  /**
   * Return the differences between two objects.
   * Properties starting with '_' will be ignored.
   * @param {*} obj
   * @param {*} refObj
   * @param {boolean=false} ignoreUnderscored
   * @return {*}
   */
  ZxUtils.objDiff = function(obj, refObj, ignoreUnderscored) {
    var p, diff = {};
    if (typeof obj === 'object') {
      for (p in obj) {
        if (ignoreUnderscored && p.charAt(0) === '_') { continue; }
        if (obj.hasOwnProperty(p) && refObj.hasOwnProperty(p)) {
          if (obj[p] && typeof obj[p] === 'object') {
            diff[p] = ZxUtils.objDiff(obj[p], refObj[p], ignoreUnderscored);
            if (ZxUtils.isObjectEmpty(diff[p])) {
              delete diff[p];
            }
          } else {
            if (obj[p] !== refObj[p]) {
              diff[p] = obj[p];
            }
          }
        }
        if (obj.hasOwnProperty(p) && !refObj.hasOwnProperty(p)) {
          diff[p] = obj[p];
        }
      }
      for (p in refObj) {
        if (ignoreUnderscored && p.charAt(0) === '_') { continue; }
        if (refObj.hasOwnProperty(p) && !obj.hasOwnProperty(p)) {
          diff[p] = null;
        }
      }
    } else {
      if (obj !== refObj) {
        return obj;
      }
    }
    return diff;
  };

  /**
   * Check if the value inserted is a number and is positive.
   * @param {string} value
   * @param {DwtEvent} event
   * @param {DwtForm} form
   * @this {XFormItem}
   */
  ZxUtils.checkNumber = function(value, event, form) {
    this.setInstanceValue(value);

    var message = 'Invalid value',
      valid = false;
    if (ZaMsg) {
      message = ZaMsg.ERROR_INVALID_VALUE;
    } else if (ZmMsg) {
      message = ZmMsg.invalidPrefValue;
    }

    if (!Number(value) && isNaN(Number(value))) {
      valid = false;
      if (this.setError) {
        this.setError(message);
      }
      if (form && form.parent && form.parent.setDirty) {
        form.parent.setDirty(false);
      }
    } else {
      if (Number(value) < 0) {
        valid = false;
        if (this.setError) {
          this.setError(message);
        }
        if (form && form.parent && form.parent.setDirty) {
          form.parent.setDirty(false);
        }
      } else {
        valid = true;
        if (this.clearError) {
          this.clearError();
        }
        if (form && form.parent && form.parent.setDirty) {
          form.parent.setDirty(true);
        }
      }
    }

    return valid;
  };

  /**
   * Return if an object is an array.
   * @param {*} obj
   * @return {boolean}
   */
  ZxUtils.isArray = function(obj) {
    return Object.prototype.toString.call(obj) == '[object Array]';
  };

  /**
   * Return if an object is an object (and not an extension).
   * @param {*} obj
   * @return {boolean}
   */
  ZxUtils.isObject = function(obj) {
    return obj === Object(obj);
  };

  /**
   * Return if an object is a function.
   * @param {*} obj
   * @return {boolean}
   */
  ZxUtils.isFunction = function(obj) {
    return Object.prototype.toString.call(obj) == '[object Function]';
  };

  /**
   * Open the given url into a new window.
   * @param {string} url
   */
  ZxUtils.openURL = function(url) {
    window.open(url, '_blank');
  };

  /**
   * Get if the current user is a delegated admin.
   * Information cached.
   * @return {boolean}
   * NOTE: Only for admin view.
   */
  ZxUtils.amIDelegated = function() {
    if (ZxUtils.onUserClient()) { return false; }
    if (ZxUtils._amIDelegated && (ZxUtils._amIDelegated === true || ZxUtils._amIDelegated === false)) {
      return ZxUtils._amIDelegated;
    }
    var currentAdmin = ZaZimbraAdmin.currentAdminAccount;

    if (!currentAdmin || !currentAdmin.attrs) {
      throw 'Unable to check account informations.';
    } else {
      if (currentAdmin.attrs.zimbraIsDelegatedAdminAccount &&
        currentAdmin.attrs.zimbraIsDelegatedAdminAccount === 'TRUE') {
        ZxUtils._amIDelegated = true;
      } else {
        ZxUtils._amIDelegated = false;
      }
    }
    return ZxUtils._amIDelegated;
  };

  /**
   * Get if the current user is NOT a delegated admin.
   * Information cached.
   * @return {boolean}
   * NOTE: Only for admin view.
   */
  ZxUtils.amINotDelegated = function() {
    if (ZxUtils.onUserClient()) { return false; }
    return !ZxUtils.amIDelegated();
  };

  /**
   * Get if the current user is NOT a delegated admin.
   * @return {boolean}
   * NOTE: Only for admin view.
   */
  ZxUtils.amIAdmin = function() {
    if (ZxUtils.onUserClient()) { return false; }
    if (ZxUtils._amIAdmin && (ZxUtils._amIAdmin === true || ZxUtils._amIAdmin === false)) {
      return ZxUtils._amIAdmin;
    }
    var currentAdmin = ZaZimbraAdmin.currentAdminAccount;

    if (!currentAdmin || !currentAdmin.attrs) {
      throw 'Unable to check account informations.';
    } else {
      if (currentAdmin.attrs.zimbraIsAdminAccount &&
        currentAdmin.attrs.zimbraIsAdminAccount === 'TRUE') {
        ZxUtils._amIAdmin = true;
      } else {
        ZxUtils._amIAdmin = false;
      }
    }
    return ZxUtils._amIAdmin;
  };

  /* ~~~~~ Backports for zimbra < 8 ~~~~~ */
  if (typeof AjxUtil !== 'undefined') {
    AjxUtil.FULL_URL_RE = AjxUtil.FULL_URL_RE || /^[A-Za-z0-9]{2,}:\/\/[A-Za-z0-9\-]+(\.[A-Za-z0-9\-]+)*(:([0-9])+)?(\/[\w\.\|\^\*\[\]\{\}\(\)\-<>~,'#_;@:!%]+)*(\/)?(\?[\w\.\|\^\*\+\[\]\{\}\(\)\-<>~,'#_;@:!%&=]*)?$/;
  }
  if (typeof Dwt !== 'undefined') {
    Dwt.moveCursorToEnd = Dwt.moveCursorToEnd || function(input) {
      if (ZxBrowserInfo.msie) {
        var tr = input.createTextRange();
        tr.moveStart('character', input.value.length);
        tr.collapse();
        tr.select();
      } else {
        input.focus();
        var length = input.value.length;
        input.setSelectionRange(length, length);
      }
    };
  }
  if (typeof DwtInputField !== 'undefined') {
    DwtInputField.prototype.moveCursorToEnd = DwtInputField.prototype.moveCursorToEnd || function() {
      Dwt.moveCursorToEnd(this._inputField);
    };
  }
  /* ~~~ End backports for zimbra < 8 ~~~ */

  /* ~~~ Laziness is over 9000! ~~~ */
  /**
   * Shortcut for the function ZxUtils.getMessage.
   * @see {ZxUtils.getMessage}
   * @param {String} messageLabel
   * @param {string[]} [substitutions] substitutions
   * @return {String}
   */
  var zxMsg = function(messageLabel, substitutions) {
    return ZxUtils.getMessage(messageLabel, substitutions);
  };
  if (typeof context.zxMsg === 'undefined') {
    context.zxMsg = zxMsg;
  }

  /**
   * Array containing all choices for an "select" select object which
   * allow the user to select an hour.
   * @type {Array}
   */
  ZxUtils.HOURS = [
    {value: '0', label: '00:00'},
    {value: '1', label: '01:00'},
    {value: '2', label: '02:00'},
    {value: '3', label: '03:00'},
    {value: '4', label: '04:00'},
    {value: '5', label: '05:00'},
    {value: '6', label: '06:00'},
    {value: '7', label: '07:00'},
    {value: '8', label: '08:00'},
    {value: '9', label: '09:00'},
    {value: '10', label: '10:00'},
    {value: '11', label: '11:00'},
    {value: '12', label: '12:00'},
    {value: '13', label: '13:00'},
    {value: '14', label: '14:00'},
    {value: '15', label: '15:00'},
    {value: '16', label: '16:00'},
    {value: '17', label: '17:00'},
    {value: '18', label: '18:00'},
    {value: '19', label: '19:00'},
    {value: '20', label: '20:00'},
    {value: '21', label: '21:00'},
    {value: '22', label: '22:00'},
    {value: '23', label: '23:00'}
  ];

  if (typeof ZaMsg !== 'undefined') {
    /**
     * Array containing all choices for an "select" select object which
     * allow the user to select a weekday, it is localized using ZaMsg strings.
     * @type {Array}
     */
    ZxUtils.WEEKDAYS = [
      {value: '1', label: ZaMsg.Monday},
      {value: '2', label: ZaMsg.Tuesday},
      {value: '3', label: ZaMsg.Wednesday},
      {value: '4', label: ZaMsg.Thursday},
      {value: '5', label: ZaMsg.Friday},
      {value: '6', label: ZaMsg.Saturday},
      {value: '7', label: ZaMsg.Sunday}
    ];
  }

  /**
   * Array containing all choices for an "select" select object which
   * allow the user to select a month, it is localized using zxMsg.
   * @type {Array}
   */
  ZxUtils.MONTHS = [
    {value: 0, label: zxMsg('LBL_January')},
    {value: 1, label: zxMsg('LBL_February')},
    {value: 2, label: zxMsg('LBL_March')},
    {value: 3, label: zxMsg('LBL_April')},
    {value: 4, label: zxMsg('LBL_May')},
    {value: 5, label: zxMsg('LBL_June')},
    {value: 6, label: zxMsg('LBL_July')},
    {value: 7, label: zxMsg('LBL_August')},
    {value: 8, label: zxMsg('LBL_September')},
    {value: 9, label: zxMsg('LBL_October')},
    {value: 10, label: zxMsg('LBL_November')},
    {value: 11, label: zxMsg('LBL_December')}
  ];

  ZxUtils.MONTHS_ARRAY = [
    zxMsg('LBL_January'),
    zxMsg('LBL_February'),
    zxMsg('LBL_March'),
    zxMsg('LBL_April'),
    zxMsg('LBL_May'),
    zxMsg('LBL_June'),
    zxMsg('LBL_July'),
    zxMsg('LBL_August'),
    zxMsg('LBL_September'),
    zxMsg('LBL_October'),
    zxMsg('LBL_November'),
    zxMsg('LBL_December')
  ];


  /**
   * Return month array <= of current month.
   * @return {Array}
   */
  ZxUtils.getMonthChoice = function() {
    var today = new Date();
    var currentMonth = today.getMonth();

    var monthArray = [];

    for (var i = 0; i < currentMonth; i++) {
      monthArray.push(ZxUtils.MONTHS[i]);
    }

    return monthArray;
  };

  /**
   *
   * @param {number} year
   * @return {Array}
   */
  ZxUtils.getYearChoiceFrom = function(year) {
    var today = new Date();
    var currentYear = today.getFullYear();

    var yearArray = [];

    for (var cYear = currentYear; cYear >= year; cYear--) {
      yearArray.push({value: cYear, label: cYear});
    }

    return yearArray;
  };

  /**
   * Get a string associated with an image.
   * @param {string=} modName
   * @param {string=} label
   * @return {string}
   */
  ZxUtils.getTitleWithIcon = function(modName, label) {
    return AjxImg.getImageHtml(modName || 'ZeXtras', 'float:left') +
      '<div style="float:left; padding-left:5px;"> ' + (label || (modName || 'ZeXtras')) + '</div>';
  };

  /**
   * Return a Clickable info icon that point to a a specified link (blank targeted)
   * @param {string} link
   * @return {string}
   */
  ZxUtils.getInfoIcon = function(link) {
    return '<a href="' + link + '" target="_blank">' +
    '<div class="ImgHelp" style="display: inline-block;"></div></a>';
  };

  /**
   * Map containing error codes used into the ZeXtras System.
   * @type {{}}
   */
  ZxUtils.ERROR_CODES = {
    CONN_1: 'Unable to connect to ZeXtras Server System',
    MISC_1: 'ZeXtras and ZxMig are installed on the same infrastructure.',
    MISC_2: 'ZeXtras Zimlet and Core versions differs.',
    MISC_3: 'Zimlet and Core are two different types.'
  };

  /**
   * Search a property inside an object.
   * @param {object} obj
   * @param {string} name
   * @param {boolean=} recursive (default false)
   * @param {string=} tempId (do not assign) Temporary id search used to avoid recoursice loops.
   * @return {*}
   */
  ZxUtils.searchAFuckingProperty = function(obj, name, recursive, tempId) {
    tempId = tempId || '_ZxUtils_walk_' + Math.random().toString(36).substring(7);
    if (typeof obj !== 'object') {
      return 'Can\'t search for a property';
    }
    if (!recursive) {
      recursive = false;
    }
    var result = {},
      reg = new RegExp(name, 'ig'),
      prop;

    if(typeof obj === 'object' && obj !== null ) {
      if (obj.hasOwnProperty(tempId)) {
        console.log('skip');
        return result;
      }
      obj[tempId] = true;
    }

    for (prop in obj) {
      if (!obj.hasOwnProperty(prop)) { continue; }

      if (prop.search(reg) > -1) {
        result[prop] = obj[prop];
      }

      if (recursive && (typeof obj[prop] === 'object')) {
        var tmpRes = ZxUtils.searchAFuckingProperty(obj[prop], name, true, tempId);
        if (!ZxUtils.isObjectEmpty(tmpRes)) {
          result[prop] = tmpRes;
        }
      }
    }

    return result;
  };

  /**
   * Display an error dialog, allowing or not the send report button.
   * @param {ZxError|Error} error
   * @param {boolean=} allowReport
   */
  ZxUtils.catchError = function(error, allowReport) {
    if (!ZxUtils.isZxDev()) { ZxUtils.enableZxDev(); }
    ZxUtils.log(error, 'Handling Error', ZxUtils.LOG_WARNING, false);
    if (!ZmCsfeCommand.noAuth) {
      ZxUtils.popupErrorDialog(error, allowReport !== false);
    }
  };

  /**
   * Search the index of an element into an array or into a String.
   * @param {string|[]} obj
   * @param {string} search
   * @param {number=} startFrom
   * @param {boolean=} strict if used to search on string, specify to search
   *                   the exact match for a substring
   * @return {number}
   */
  ZxUtils.indexOf = function(obj, search, startFrom, strict) {
    strict = strict || false;
    var pos, i;
    if (typeof obj === 'string') {
      pos = obj.indexOf(search, startFrom || 0);
      if (pos < 0 || !strict) {
        return pos;
      } else {
        for (i = 0; i < search.length; i += 1) {
          if (obj[pos + i] !== search[i]) {
            return -1;
          }
        }
        return pos;
      }
    } else if (ZxUtils.isArray(obj)) {
      return AjxUtil.indexOf(obj, search);
    } else {
      throw new TypeError('[obj] object type not handled: ' + typeof obj);
    }
  };

  /**
   * Search an injection point into an XForm object to modify it.
   * @param {object} xForm
   * @param {string} entrypoint
   * @return {object}
   */
  ZxUtils.searchXFormInjectionPoint = function(xForm, entrypoint) {
    if (!xForm || !xForm.items) { return; }
    var i = 0, tmp;

    for (i = 0; i < xForm.items.length; i += 1) {
      if (xForm.items[i].id && xForm.items[i].id === entrypoint) { return xForm.items; }
      tmp = ZxUtils.searchXFormInjectionPoint(xForm.items[i], entrypoint);
      if (tmp) { return tmp; }
    }
  };

  /**
   * ZeXtras version of {@see AjxDateUtil.computeWordyDateStr}.
   * @param {Date} now used to display messages like "Today, Yesterday"
   * @param {Number} dateMSec will be converted into Date
   * @return {String}
   */
  ZxUtils.computeWordyDateStr = function(now, dateMSec) {
    if (dateMSec == null) { return ''; }

    var date = new Date(dateMSec),
      normalizedNow = new Date(
        now.getFullYear(),
        now.getMonth(),
        now.getDate(),
        0,
        0,
        0
      ),
      yesterday = new Date(normalizedNow.getTime() - ZxUtils.MSEC_PER_DAY),
      tomorrow = new Date(normalizedNow.getTime() + ZxUtils.MSEC_PER_DAY);

    if (normalizedNow.getTime() <= date.getTime() && date.getTime() < tomorrow.getTime()) {
      // Today
      if (!ZxUtils._wordyDateToday) {
        ZxUtils._wordyDateToday = new AjxDateFormat(AjxMsg.formatWordyDateToday);
      }
      return ZxUtils._wordyDateToday.format(date);

    } else if (yesterday.getTime() <= date.getTime() && date.getTime() < normalizedNow.getTime()) {
      // Yesterday
      if (!ZxUtils._wordyDateYesterday) {
        ZxUtils._wordyDateYesterday = new AjxDateFormat(AjxMsg.formatWordyDateYesterday);
      }
      return ZxUtils._wordyDateYesterday.format(date);

    } else {
      // Generic date
      if (!ZxUtils._wordyDate) {
        ZxUtils._wordyDate = new AjxDateFormat(AjxMsg.formatWordyDate);
      }
      return ZxUtils._wordyDate.format(date);
    }
  };

  /**
   * Sleep for a given amount of milliseconds.
   * @param {number} milliseconds
   */
  ZxUtils.sleep = function(milliseconds) {
    var start = new Date().getTime();
    while (true) {
      if ((new Date().getTime() - start) > milliseconds) {
        break;
      }
    }
  };

  /**
   * Get the name of a function.
   * Surprise! IE Does not support the Function.prototype.name property!
   * @param {function|string} fcn
   * @return {string} the function name, empty string if is a unnamed funciton.
   */
  ZxUtils.getFunctionName = function(fcn) {
    if (typeof fcn === 'string') {
      return fcn;
    }
    if (typeof Function.prototype.name === 'string' && typeof fcn === 'function') {
      return fcn.name;
    }
    var funcNameRegex = /function\s([^(]{1,})\(/,
      results = funcNameRegex.exec(fcn.toString());
    return (results && results.length > 1) ? results[1].trim() : '';
  };

  context.ZxUtils = ZxUtils;
}(window));

if (typeof module !== 'undefined' && module.exports) {
  module.exports = window.ZxUtils;
}
